@@ -21,16 +21,17 @@ from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (NoSuchFile, DivergedBranches, NoSuchRevision, 
                            NotBranchError)
 from bzrlib.inventory import (Inventory)
+from bzrlib.revision import ensure_null, NULL_REVISION
 from bzrlib.workingtree import WorkingTree
 
 import svn.client, svn.core
-from svn.core import SubversionException
+from svn.core import SubversionException, Pool
 
 from commit import push
 from errors import NotSvnBranchPath
 from format import get_rich_root_format
 from repository import SvnRepository
-from transport import bzr_to_svn_url, svn_config
+from transport import bzr_to_svn_url, create_svn_client
 
 
 class FakeControlFiles(object):
@@ -70,8 +71,9 @@ class SvnBranch(Branch):
         self._cached_revnum = None
         self._revision_history = None
         self._revision_history_revnum = None
-        self.scheme = self.repository.get_scheme()
+        self.mapping = self.repository.get_mapping()
         self._branch_path = branch_path.strip("/")
+        assert isinstance(self._branch_path, str)
         try:
             if self.repository.transport.check_path(branch_path.strip("/"), 
                 self.get_revnum()) != svn.core.svn_node_dir:
@@ -80,9 +82,8 @@ class SvnBranch(Branch):
             if num == svn.core.SVN_ERR_FS_NO_SUCH_REVISION:
                 raise NotBranchError(self.base)
             raise
-        if (not self.scheme.is_branch(branch_path) and 
-            not self.scheme.is_tag(branch_path)):
-            raise NotSvnBranchPath(branch_path, scheme=self.scheme)
+        if not self.mapping.is_branch(branch_path):
+            raise NotSvnBranchPath(branch_path, scheme=self.mapping.scheme)
 
     def set_branch_path(self, branch_path):
         """Change the branch path for this branch.
@@ -91,6 +92,14 @@ class SvnBranch(Branch):
         """
         self._branch_path = branch_path.strip("/")
 
+    def unprefix(self, relpath):
+        """Remove the branch path from a relpath.
+
+        :param relpath: path from the repository root.
+        """
+        assert relpath.startswith(self.get_branch_path())
+        return relpath[len(self.get_branch_path()):].strip("/")
+
     def get_branch_path(self, revnum=None):
         """Find the branch path of this branch in the specified revnum.
 
@@ -120,11 +129,12 @@ class SvnBranch(Branch):
         """
         return BranchCheckResult(self)
 
-    def _create_heavyweight_checkout(self, to_location, revision_id=None):
+    def _create_heavyweight_checkout(self, to_location, revision_id=None, hardlink=False):
         """Create a new heavyweight checkout of this branch.
 
         :param to_location: URL of location to create the new checkout in.
         :param revision_id: Revision that should be the tip of the checkout.
+        :param hardlink: Whether to hardlink
         :return: WorkingTree object of checkout.
         """
         checkout_branch = BzrDir.create_branch_convenience(
@@ -134,7 +144,7 @@ class SvnBranch(Branch):
         # pull up to the specified revision_id to set the initial 
         # branch tip correctly, and seed it with history.
         checkout_branch.pull(self, stop_revision=revision_id)
-        return checkout.create_workingtree(revision_id)
+        return checkout.create_workingtree(revision_id, hardlink=hardlink)
 
     def lookup_revision_id(self, revid):
         """Look up the matching Subversion revision number on the mainline of 
@@ -144,8 +154,8 @@ class SvnBranch(Branch):
         :return: Revision number on the branch. 
         :raises NoSuchRevision: If the revision id was not found.
         """
-        (bp, revnum, scheme) = self.repository.lookup_revision_id(revid, 
-                                                             scheme=self.scheme)
+        (bp, revnum, mapping) = self.repository.lookup_revision_id(revid, 
+                                                             scheme=self.mapping.scheme)
         assert bp.strip("/") == self.get_branch_path(revnum).strip("/"), \
                 "Got %r, expected %r" % (bp, self.get_branch_path(revnum))
         return revnum
@@ -168,37 +178,31 @@ class SvnBranch(Branch):
             rev.kind = svn.core.svn_opt_revision_number
             rev.value.number = revnum
 
-        client_ctx = svn.client.create_context()
-        client_ctx.config = svn_config
+        client_ctx = create_svn_client(Pool())
         svn.client.checkout(bzr_to_svn_url(self.base), to_location, rev, 
                             True, client_ctx)
 
         return WorkingTree.open(to_location)
 
-    def create_checkout(self, to_location, revision_id=None, lightweight=False):
+    def create_checkout(self, to_location, revision_id=None, lightweight=False,
+                        accelerator_tree=None, hardlink=False):
         """See Branch.create_checkout()."""
         if lightweight:
             return self._create_lightweight_checkout(to_location, revision_id)
         else:
-            return self._create_heavyweight_checkout(to_location, revision_id)
+            return self._create_heavyweight_checkout(to_location, revision_id, hardlink=hardlink)
 
     def generate_revision_id(self, revnum):
         """Generate a new revision id for a revision on this branch."""
         assert isinstance(revnum, int)
         return self.repository.generate_revision_id(
-                revnum, self.get_branch_path(revnum), str(self.scheme))
+                revnum, self.get_branch_path(revnum), self.mapping)
        
     def _generate_revision_history(self, last_revnum):
         """Generate the revision history up until a specified revision."""
-        self._revision_history = []
-        for (branch, rev) in self.repository.follow_branch(
-                self.get_branch_path(last_revnum), last_revnum, self.scheme):
-            self._revision_history.append(
-                self.repository.generate_revision_id(rev, branch, 
-                    str(self.scheme)))
-        self._revision_history.reverse()
-        self._revision_history_revnum = last_revnum
-        self.repository.revmap.insert_revision_history(self._revision_history)
+        revhistory = list(self.repository.iter_lhs_ancestry(self.generate_revision_id(last_revnum)))
+        revhistory.reverse()
+        return revhistory
 
     def _get_nick(self):
         """Find the nick name for this branch.
@@ -224,6 +228,13 @@ class SvnBranch(Branch):
         last_revid = self.last_revision()
         return self.revision_id_to_revno(last_revid), last_revid
 
+    def get_root_id(self, revnum=None):
+        if revnum is None:
+            tree = self.basis_tree()
+        else:
+            tree = self.repository.revision_tree(self.get_rev_id(revnum))
+        return tree.get_root_id()
+
     def revno(self):
         """See Branch.revno()."""
         return self.last_revision_info()[0]
@@ -256,7 +267,9 @@ class SvnBranch(Branch):
             last_revnum = self.get_revnum()
         if (self._revision_history is None or 
             self._revision_history_revnum != last_revnum):
-            self._generate_revision_history(last_revnum)
+            self._revision_history = self._generate_revision_history(last_revnum)
+            self._revision_history_revnum = last_revnum
+            self.repository.revmap.insert_revision_history(self._revision_history)
         return self._revision_history
 
     def last_revision(self):
@@ -267,16 +280,16 @@ class SvnBranch(Branch):
         if (self._revision_history is None or 
             self._revision_history_revnum != last_revnum):
             for (branch, rev) in self.repository.follow_branch(
-                self.get_branch_path(), last_revnum, self.scheme):
+                self.get_branch_path(), last_revnum, self.mapping):
                 return self.repository.generate_revision_id(rev, branch, 
-                                                            str(self.scheme))
-            return None
+                                                            self.mapping)
+            return NULL_REVISION
 
         ph = self.revision_history(last_revnum)
         if ph:
             return ph[-1]
         else:
-            return None
+            return NULL_REVISION
 
     def pull(self, source, overwrite=False, stop_revision=None, 
              _hook_master=None, run_hooks=True):
@@ -333,26 +346,27 @@ class SvnBranch(Branch):
 
     def update_revisions(self, other, stop_revision=None):
         """See Branch.update_revisions()."""
+        if stop_revision is None:
+            stop_revision = ensure_null(other.last_revision())
         if (self.last_revision() == stop_revision or
             self.last_revision() == other.last_revision()):
             return
-        if isinstance(other, SvnBranch) and \
-            other.repository.uuid == self.repository.uuid:
-            # FIXME: Make sure branches haven't diverged
-            # FIXME: svn.ra.del_dir(self.base_path)
-            # FIXME: svn.ra.copy_dir(other.base_path, self.base_path)
-            raise NotImplementedError(self.pull)
-        else:
-            # TODO: Use stop_revision
-            todo = self.missing_revisions(other)
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                for rev_id in todo:
-                    pb.update("pushing revisions", todo.index(rev_id), 
-                              len(todo))
-                    push(self, other, rev_id)
-            finally:
-                pb.finished()
+        if not other.repository.get_graph().is_ancestor(self.last_revision(), 
+                                                        stop_revision):
+            if self.repository.get_graph().is_ancestor(stop_revision, 
+                                                       self.last_revision()):
+                return
+            raise DivergedBranches(self, other)
+        todo = self.repository.lhs_missing_revisions(other.revision_history(), 
+                                                     stop_revision)
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for revid in todo:
+                pb.update("pushing revisions", todo.index(revid), 
+                          len(todo))
+                push(self, other, revid)
+        finally:
+            pb.finished()
 
     def lock_write(self):
         """See Branch.lock_write()."""
@@ -382,7 +396,7 @@ class SvnBranch(Branch):
 
     def get_parent(self):
         """See Branch.get_parent()."""
-        return self.base
+        return None
 
     def set_parent(self, url):
         """See Branch.set_parent()."""
@@ -416,8 +430,8 @@ class SvnBranchFormat(BranchFormat):
 
     def __get_matchingbzrdir(self):
         """See BranchFormat.__get_matchingbzrdir()."""
-        from format import SvnFormat
-        return SvnFormat()
+        from remote import SvnRemoteFormat
+        return SvnRemoteFormat()
 
     _matchingbzrdir = property(__get_matchingbzrdir)
 
