@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2007 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2006-2008 Jelmer Vernooij <jelmer@samba.org>
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,25 +16,40 @@
 """Committing and pushing to Subversion repositories."""
 
 import svn.delta
-from svn.core import Pool, SubversionException
+from svn.core import Pool, SubversionException, svn_time_to_cstring
 
 from bzrlib import debug, osutils, urlutils
 from bzrlib.branch import Branch
-from bzrlib.errors import InvalidRevisionId, DivergedBranches, UnrelatedBranches
+from bzrlib.errors import (BzrError, InvalidRevisionId, DivergedBranches, 
+                           UnrelatedBranches)
 from bzrlib.inventory import Inventory
 from bzrlib.repository import RootCommitBuilder, InterRepository
 from bzrlib.revision import NULL_REVISION
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, warning
 
-from copy import deepcopy
-from repository import (SVN_PROP_BZR_ANCESTRY, SVN_PROP_BZR_FILEIDS,
-                        SVN_PROP_SVK_MERGE, SVN_PROP_BZR_REVISION_INFO, 
-                        SVN_PROP_BZR_REVISION_ID, revision_id_to_svk_feature,
-                        generate_revision_metadata, SvnRepositoryFormat, 
-                        SvnRepository)
+from cStringIO import StringIO
+from errors import ChangesRootLHSHistory, MissingPrefix, RevpropChangeFailed
+from svk import (generate_svk_feature, serialize_svk_features, 
+                 parse_svk_features, SVN_PROP_SVK_MERGE)
+from mapping import parse_revision_id
+from repository import (SvnRepositoryFormat, SvnRepository, lazy_dict)
+from scheme import is_valid_property_name
 import urllib
 
 
+def _revision_id_to_svk_feature(revid):
+    """Create a SVK feature identifier from a revision id.
+
+    :param revid: Revision id to convert.
+    :return: Matching SVK feature identifier.
+    """
+    assert isinstance(revid, str)
+    (uuid, branch, revnum, _) = parse_revision_id(revid)
+    # TODO: What about renamed revisions? Should use 
+    # repository.lookup_revision_id here.
+    return generate_svk_feature(uuid, branch, revnum)
+
+
 def _check_dirs_exist(transport, bp_parts, base_rev):
     """Make sure that the specified directories exist.
 
@@ -46,11 +61,27 @@ def _check_dirs_exist(transport, bp_parts, base_rev):
     """
     for i in range(len(bp_parts), 0, -1):
         current = bp_parts[:i]
-        if transport.check_path("/".join(current).strip("/"), base_rev) == svn.core.svn_node_dir:
+        path = "/".join(current).strip("/")
+        if transport.check_path(path, base_rev) == svn.core.svn_node_dir:
             return current
     return []
 
 
+def set_svn_revprops(transport, revnum, revprops):
+    """Attempt to change the revision properties on the
+    specified revision.
+
+    :param transport: SvnRaTransport connected to target repository
+    :param revnum: Revision number of revision to change metadata of.
+    :param revprops: Dictionary with revision properties to set.
+    """
+    for (name, value) in revprops.items():
+        try:
+            transport.change_rev_prop(revnum, name, value)
+        except SubversionException, (_, svn.core.SVN_ERR_REPOS_DISABLED_FEATURE):
+            raise RevpropChangeFailed(name)
+
+
 class SvnCommitBuilder(RootCommitBuilder):
     """Commit Builder implementation wrapped around svn_delta_editor. """
 
@@ -75,38 +106,22 @@ class SvnCommitBuilder(RootCommitBuilder):
         self.branch = branch
         self.pool = Pool()
 
-        # Keep track of what Subversion properties to set later on
-        self._svnprops = {}
-        self._svnprops[SVN_PROP_BZR_REVISION_INFO] = generate_revision_metadata(
-            timestamp, timezone, committer, revprops)
-        self._svnprops[SVN_PROP_BZR_FILEIDS] = ""
-
         # Gather information about revision on top of which the commit is 
         # happening
-        (self.base_revno, self.base_revid) = self.branch.last_revision_info()
+        if parents == []:
+            self.base_revid = None
+        else:
+            self.base_revid = parents[0]
+
+        self.base_revno = self.branch.revision_id_to_revno(self.base_revid)
         if self.base_revid is None:
             self.base_revnum = -1
             self.base_path = None
-            self.base_scheme = repository.get_scheme()
+            self.base_mapping = repository.get_mapping()
         else:
-            (self.base_path, self.base_revnum, self.base_scheme) = \
+            (self.base_path, self.base_revnum, self.base_mapping) = \
                 repository.lookup_revision_id(self.base_revid)
 
-        # Determine revisions merged in this one
-        merges = filter(lambda x: x != self.base_revid, parents)
-
-        if len(merges) > 0:
-            self._record_merges(merges)
-
-        # Set appropriate property if revision id was specified by 
-        # caller
-        if revision_id is not None:
-            self._record_revision_id(revision_id)
-
-        # At least one of the parents has to be the last revision on the 
-        # mainline in Subversion.
-        assert (self.base_revid is None or self.base_revid in parents)
-
         if old_inv is None:
             if self.base_revid is None:
                 self.old_inv = Inventory(root_id=None)
@@ -118,59 +133,35 @@ class SvnCommitBuilder(RootCommitBuilder):
             # so allow None as well.
             assert self.old_inv.revision_id in (None, self.base_revid)
 
+        # Determine revisions merged in this one
+        merges = filter(lambda x: x != self.base_revid, parents)
+
         self.modified_files = {}
         self.modified_dirs = set()
+        if self.base_revid is None:
+            base_branch_props = {}
+        else:
+            base_branch_props = lazy_dict(lambda: self.repository.branchprop_list.get_properties(self.base_path, self.base_revnum))
+        (self._svn_revprops, self._svnprops) = self.base_mapping.export_revision(self.branch.get_branch_path(), timestamp, timezone, committer, revprops, revision_id, self.base_revno+1, merges, base_branch_props)
+
+        if len(merges) > 0:
+            old_svk_features = parse_svk_features(base_branch_props.get(SVN_PROP_SVK_MERGE, ""))
+            svk_features = set(old_svk_features)
+
+            # SVK compatibility
+            for merge in merges:
+                try:
+                    svk_features.add(_revision_id_to_svk_feature(merge))
+                except InvalidRevisionId:
+                    pass
+
+            if old_svk_features != svk_features:
+                self._svnprops[SVN_PROP_SVK_MERGE] = serialize_svk_features(svk_features)
 
     def mutter(self, text):
         if 'commit' in debug.debug_flags:
             mutter(text)
 
-    def _record_revision_id(self, revid):
-        """Store the revision id in a file property.
-
-        :param revid: The revision id.
-        """
-        if self.base_revid is not None:
-            old = self.repository.branchprop_list.get_property(
-                    self.base_path, self.base_revnum, 
-                        SVN_PROP_BZR_REVISION_ID+str(self.base_scheme), "")
-        else:
-            old = ""
-
-        self._svnprops[SVN_PROP_BZR_REVISION_ID+str(self.base_scheme)] = \
-                old + "%d %s\n" % (self.base_revno+1, revid)
-
-    def _record_merges(self, merges):
-        """Store the extra merges (non-LHS parents) in a file property.
-
-        :param merges: List of parents.
-        """
-        # Bazaar Parents
-        if self.base_revid is not None:
-            old = self.repository.branchprop_list.get_property(
-                  self.base_path, self.base_revnum, 
-                  SVN_PROP_BZR_ANCESTRY+str(self.base_scheme), "")
-        else:
-            old = ""
-        self._svnprops[SVN_PROP_BZR_ANCESTRY+str(self.base_scheme)] = old + "\t".join(merges) + "\n"
-
-        if self.base_revid is not None:
-            old = self.repository.branchprop_list.get_property(
-                self.base_path, self.base_revnum, SVN_PROP_SVK_MERGE, "")
-        else:
-            old = ""
-
-        new = ""
-        # SVK compatibility
-        for merge in merges:
-            try:
-                new += "%s\n" % revision_id_to_svk_feature(merge)
-            except InvalidRevisionId:
-                pass
-
-        if new != "":
-            self._svnprops[SVN_PROP_SVK_MERGE] = old + new
-        
     def _generate_revision_if_needed(self):
         """See CommitBuilder._generate_revision_if_needed()."""
 
@@ -201,13 +192,11 @@ class SvnCommitBuilder(RootCommitBuilder):
         :param baton: Baton under which the file is known to the editor.
         """
         assert baton is not None
-        if contents == "" and not file_id in self.old_inv:
-            # Don't send diff if a new file with empty contents is 
-            # added, because it created weird exceptions over svn+ssh:// 
-            # or https://
-            return
         (txdelta, txbaton) = self.editor.apply_textdelta(baton, None, self.pool)
-        svn.delta.svn_txdelta_send_string(contents, txdelta, txbaton, self.pool)
+        digest = svn.delta.svn_txdelta_send_stream(StringIO(contents), txdelta, txbaton, self.pool)
+        if 'validate' in debug.debug_flags:
+            from fetch import md5_strings
+            assert digest == md5_strings(contents)
 
     def _dir_process(self, path, file_id, baton):
         """Pass the changes to a directory to the commit editor.
@@ -233,9 +222,8 @@ class SvnCommitBuilder(RootCommitBuilder):
                     self.new_inventory[child_ie.file_id].name != child_name):
                     self.mutter('removing %r(%r)' % (child_name, child_ie.file_id))
                     self.editor.delete_entry(
-                            urlutils.join(
-                                self.branch.get_branch_path(), path, child_name), 
-                            self.base_revnum, baton, self.pool)
+                        urlutils.join(self.branch.get_branch_path(), path, child_name), 
+                        self.base_revnum, baton, self.pool)
 
         # Loop over file children of file_id in self.new_inventory
         for child_name in self.new_inventory[file_id].children:
@@ -249,18 +237,17 @@ class SvnCommitBuilder(RootCommitBuilder):
             # add them if they didn't exist in old_inv 
             if not child_ie.file_id in self.old_inv:
                 self.mutter('adding %s %r' % (child_ie.kind, new_child_path))
-                self._record_file_id(child_ie, new_child_path)
                 child_baton = self.editor.add_file(
                     urlutils.join(self.branch.get_branch_path(), 
                                   new_child_path), baton, None, -1, self.pool)
 
 
             # copy if they existed at different location
-            elif self.old_inv.id2path(child_ie.file_id) != new_child_path:
+            elif (self.old_inv.id2path(child_ie.file_id) != new_child_path or
+                    self.old_inv[child_ie.file_id].parent_id != child_ie.parent_id):
                 self.mutter('copy %s %r -> %r' % (child_ie.kind, 
                                   self.old_inv.id2path(child_ie.file_id), 
                                   new_child_path))
-                self._record_file_id(child_ie, new_child_path)
                 child_baton = self.editor.add_file(
                     urlutils.join(self.branch.get_branch_path(), new_child_path), baton, 
                     urlutils.join(self.repository.transport.svn_url, self.base_path, self.old_inv.id2path(child_ie.file_id)),
@@ -323,7 +310,6 @@ class SvnCommitBuilder(RootCommitBuilder):
             # add them if they didn't exist in old_inv 
             if not child_ie.file_id in self.old_inv:
                 self.mutter('adding dir %r' % child_ie.name)
-                self._record_file_id(child_ie, new_child_path)
                 child_baton = self.editor.add_directory(
                     urlutils.join(self.branch.get_branch_path(), 
                                   new_child_path), baton, None, -1, self.pool)
@@ -332,7 +318,6 @@ class SvnCommitBuilder(RootCommitBuilder):
             elif self.old_inv.id2path(child_ie.file_id) != new_child_path:
                 old_child_path = self.old_inv.id2path(child_ie.file_id)
                 self.mutter('copy dir %r -> %r' % (old_child_path, new_child_path))
-                self._record_file_id(child_ie, new_child_path)
                 child_baton = self.editor.add_directory(
                     urlutils.join(self.branch.get_branch_path(), new_child_path),
                     baton, 
@@ -357,7 +342,7 @@ class SvnCommitBuilder(RootCommitBuilder):
             self.editor.close_directory(child_baton, self.pool)
 
     def open_branch_batons(self, root, elements, existing_elements, 
-                           base_path, base_rev):
+                           base_path, base_rev, replace_existing):
         """Open a specified directory given a baton for the repository root.
 
         :param root: Baton for the repository root
@@ -365,6 +350,7 @@ class SvnCommitBuilder(RootCommitBuilder):
         :param existing_elements: List of directory names that exist
         :param base_path: Path to base top-level branch on
         :param base_rev: Revision of path to base top-level branch on
+        :param replace_existing: Whether the current branch should be replaced
         """
         ret = [root]
 
@@ -377,8 +363,9 @@ class SvnCommitBuilder(RootCommitBuilder):
             ret.append(self.editor.open_directory(
                 "/".join(existing_elements[0:i+1]), ret[-1], -1, self.pool))
 
-        assert (len(existing_elements) == len(elements) or 
-                len(existing_elements)+1 == len(elements))
+        if (len(existing_elements) != len(elements) and
+            len(existing_elements)+1 != len(elements)):
+            raise MissingPrefix("/".join(elements))
 
         # Branch already exists and stayed at the same location, open:
         # TODO: What if the branch didn't change but the new revision 
@@ -386,19 +373,24 @@ class SvnCommitBuilder(RootCommitBuilder):
         # This needs to also check that base_rev was the latest version of 
         # branch_path.
         if (len(existing_elements) == len(elements) and 
-            base_path.strip("/") == "/".join(elements).strip("/")):
+            not replace_existing):
             ret.append(self.editor.open_directory(
                 "/".join(elements), ret[-1], base_rev, self.pool))
         else: # Branch has to be created
             # Already exists, old copy needs to be removed
-            if len(existing_elements) == len(elements):
-                self.editor.delete_entry("/".join(elements), -1, ret[-1])
+            name = "/".join(elements)
+            if replace_existing:
+                if name == "":
+                    raise ChangesRootLHSHistory()
+                self.mutter("removing branch dir %r" % name)
+                self.editor.delete_entry(name, -1, ret[-1])
             if base_path is not None:
                 base_url = urlutils.join(self.repository.transport.svn_url, base_path)
             else:
                 base_url = None
+            self.mutter("adding branch dir %r" % name)
             ret.append(self.editor.add_directory(
-                "/".join(elements), ret[-1], base_url, base_rev, self.pool))
+                name, ret[-1], base_url, base_rev, self.pool))
 
         return ret
 
@@ -406,48 +398,98 @@ class SvnCommitBuilder(RootCommitBuilder):
         """Finish the commit.
 
         """
-        def done(revision, date, author):
+        def done(revision_data, pool):
             """Callback that is called by the Subversion commit editor 
             once the commit finishes.
 
-            :param revision: Revision number
-            :param date: Date recorded for this commit
+            :param revision_data: Revision metadata
             """
-            assert revision > 0
-            self.revnum = revision
-            self.date = date
-            self.author = author
+            self.revision_metadata = revision_data
         
         bp_parts = self.branch.get_branch_path().split("/")
+        repository_latest_revnum = self.repository.transport.get_latest_revnum()
         lock = self.repository.transport.lock_write(".")
+        set_revprops = self.repository.get_config().get_set_revprops()
+        remaining_revprops = self._svn_revprops # Keep track of the revprops that haven't been set yet
+
+        # Store file ids
+        def _dir_process_file_id(old_inv, new_inv, path, file_id):
+            ret = []
+            for child_name in new_inv[file_id].children:
+                child_ie = new_inv.get_child(file_id, child_name)
+                new_child_path = new_inv.id2path(child_ie.file_id)
+                assert child_ie is not None
+
+                if (not child_ie.file_id in old_inv or 
+                    old_inv.id2path(child_ie.file_id) != new_child_path or
+                    old_inv[child_ie.file_id].parent_id != child_ie.parent_id):
+                    ret.append((child_ie.file_id, new_child_path))
+
+                if (child_ie.kind == 'directory' and 
+                    child_ie.file_id in self.modified_dirs):
+                    ret += _dir_process_file_id(old_inv, new_inv, new_child_path, child_ie.file_id)
+            return ret
+
+        fileids = {}
+
+        if (self.old_inv.root is None or 
+            self.new_inventory.root.file_id != self.old_inv.root.file_id):
+            fileids[""] = self.new_inventory.root.file_id
+
+        for id, path in _dir_process_file_id(self.old_inv, self.new_inventory, "", self.new_inventory.root.file_id):
+            fileids[path] = id
+
+        self.base_mapping.export_fileid_map(fileids, self._svn_revprops, self._svnprops)
+        self._svn_revprops[svn.core.SVN_PROP_REVISION_LOG] = message.encode("utf-8")
 
         try:
             existing_bp_parts = _check_dirs_exist(self.repository.transport, 
                                               bp_parts, -1)
-            self.revnum = None
-            self.editor = self.repository.transport.get_commit_editor(
-                message.encode("utf-8"), done, None, False)
+            self.revision_metadata = None
+            for prop in self._svn_revprops:
+                if not is_valid_property_name(prop):
+                    warning("Setting property %r with invalid characters in name" % prop)
+            try:
+                self.editor = self.repository.transport.get_commit_editor(
+                        self._svn_revprops, done, None, False)
+                self._svn_revprops = {}
+            except NotImplementedError:
+                if set_revprops:
+                    raise
+                # Try without bzr: revprops
+                self.editor = self.repository.transport.get_commit_editor({
+                    svn.core.SVN_PROP_REVISION_LOG: self._svn_revprops[svn.core.SVN_PROP_REVISION_LOG]},
+                    done, None, False)
+                del self._svn_revprops[svn.core.SVN_PROP_REVISION_LOG]
 
             root = self.editor.open_root(self.base_revnum)
-            
-            # TODO: Accept create_prefix argument
-            branch_batons = self.open_branch_batons(root, bp_parts,
-                existing_bp_parts, self.base_path, self.base_revnum)
 
-            # Make sure the root id is stored properly
-            if (self.old_inv.root is None or 
-                self.new_inventory.root.file_id != self.old_inv.root.file_id):
-                self._record_file_id(self.new_inventory.root, "")
+            replace_existing = False
+            # See whether the base of the commit matches the lhs parent
+            # if not, we need to replace the existing directory
+            if len(bp_parts) == len(existing_bp_parts):
+                if self.base_path.strip("/") != "/".join(bp_parts).strip("/"):
+                    replace_existing = True
+                elif self.base_revnum < self.repository._log.find_latest_change(self.branch.get_branch_path(), repository_latest_revnum, include_children=True):
+                    replace_existing = True
+
+            # TODO: Accept create_prefix argument (#118787)
+            branch_batons = self.open_branch_batons(root, bp_parts,
+                existing_bp_parts, self.base_path, self.base_revnum, 
+                replace_existing)
 
             self._dir_process("", self.new_inventory.root.file_id, 
                 branch_batons[-1])
 
             # Set all the revprops
             for prop, value in self._svnprops.items():
+                if not is_valid_property_name(prop):
+                    warning("Setting property %r with invalid characters in name" % prop)
                 if value is not None:
                     value = value.encode('utf-8')
                 self.editor.change_dir_prop(branch_batons[-1], prop, value, 
                                             self.pool)
+                self.mutter("Setting root file property %r -> %r" % (prop, value))
 
             for baton in reversed(branch_batons):
                 self.editor.close_directory(baton, self.pool)
@@ -456,30 +498,36 @@ class SvnCommitBuilder(RootCommitBuilder):
         finally:
             lock.unlock()
 
-        assert self.revnum is not None
+        assert self.revision_metadata is not None
 
         # Make sure the logwalker doesn't try to use ra 
         # during checkouts...
-        self.repository._log.fetch_revisions(self.revnum)
+        self.repository._log.fetch_revisions(self.revision_metadata.revision)
 
-        revid = self.branch.generate_revision_id(self.revnum)
+        revid = self.branch.generate_revision_id(self.revision_metadata.revision)
 
         assert self._new_revision_id is None or self._new_revision_id == revid
 
         self.mutter('commit %d finished. author: %r, date: %r, revid: %r' % 
-               (self.revnum, self.author, self.date, revid))
+               (self.revision_metadata.revision, self.revision_metadata.author, 
+                   self.revision_metadata.date, revid))
+
+        if self.repository.get_config().get_override_svn_revprops():
+            set_svn_revprops(self.repository.transport, 
+                 self.revision_metadata.revision, {
+                svn.core.SVN_PROP_REVISION_AUTHOR: self._committer,
+                svn.core.SVN_PROP_REVISION_DATE: svn_time_to_cstring(1000000*self._timestamp)})
+
+        try:
+            set_svn_revprops(self.repository.transport, self.revision_metadata.revision, 
+                         self._svn_revprops) 
+        except RevpropChangeFailed:
+            pass # Ignore for now
 
         return revid
 
-    def _record_file_id(self, ie, path):
-        """Store the file id of an inventory entry in a file property.
-
-        :param ie: Inventory entry.
-        :param path: Path of the inventory entry.
-        """
-        self._svnprops[SVN_PROP_BZR_FILEIDS] += "%s\t%s\n" % (urllib.quote(path), ie.file_id)
-
-    def record_entry_contents(self, ie, parent_invs, path, tree):
+    def record_entry_contents(self, ie, parent_invs, path, tree,
+                              content_summary):
         """Record the content of ie from tree into the commit if needed.
 
         Side effect: sets ie.revision when unchanged
@@ -489,24 +537,14 @@ class SvnCommitBuilder(RootCommitBuilder):
             commit.
         :param path: The path the entry is at in the tree.
         :param tree: The tree which contains this entry and should be used to 
-        obtain content.
+            obtain content.
+        :param content_summary: Summary data from the tree about the paths
+                content - stat, length, exec, sha/link target. This is only
+                accessed when the entry has a revision of None - that is when 
+                it is a candidate to commit.
         """
-        assert self.new_inventory.root is not None or ie.parent_id is None
         self.new_inventory.add(ie)
 
-        # ie.revision is always None if the InventoryEntry is considered
-        # for committing. ie.snapshot will record the correct revision 
-        # which may be the sole parent if it is untouched.
-        if ie.revision is not None:
-            return
-
-        previous_entries = ie.find_previous_heads(parent_invs, 
-            self.repository.weave_store, None)
-
-        # we are creating a new revision for ie in the history store
-        # and inventory.
-        ie.snapshot(self._new_revision_id, path, previous_entries, tree, self)
-
 
 def replay_delta(builder, old_tree, new_tree):
     """Replays a delta to a commit builder.
@@ -515,6 +553,10 @@ def replay_delta(builder, old_tree, new_tree):
     :param old_tree: Original tree on top of which the delta should be applied
     :param new_tree: New tree that should be committed
     """
+    for path, ie in new_tree.inventory.iter_entries():
+        builder.record_entry_contents(ie.copy(), [old_tree.inventory], 
+                                      path, new_tree, None)
+    builder.finish_inventory()
     delta = new_tree.changes_from(old_tree)
     def touch_id(id):
         ie = builder.new_inventory[id]
@@ -528,7 +570,8 @@ def replay_delta(builder, old_tree, new_tree):
                 builder.modified_directory(id, [])
             id = builder.new_inventory[id].parent_id
 
-        if ie.kind == 'link':
+        assert ie.kind in ('symlink', 'file', 'directory')
+        if ie.kind == 'symlink':
             builder.modified_link(ie.file_id, [], ie.symlink_target)
         elif ie.kind == 'file':
             def get_text():
@@ -552,60 +595,9 @@ def replay_delta(builder, old_tree, new_tree):
         if old_parent_id in builder.new_inventory:
             touch_id(old_parent_id)
 
-    builder.finish_inventory()
-
-
-def push_as_merged(target, source, revision_id):
-    """Push a revision as merged revision.
-
-    This will create a new revision in the target repository that 
-    merges the specified revision but does not contain any other differences. 
-    This is done so that the revision that is being pushed does not need 
-    to completely match the target revision and so it can not have the 
-    same revision id.
-
-    :param target: Repository to push to
-    :param source: Repository to pull the revision from
-    :param revision_id: Revision id of the revision to push
-    :return: The revision id of the created revision
-    """
-    assert isinstance(source, Branch)
-    rev = source.repository.get_revision(revision_id)
-
-    # revision on top of which to commit
-    prev_revid = target.last_revision()
-
-    mutter('committing %r on top of %r' % (revision_id, prev_revid))
-
-    old_tree = source.repository.revision_tree(revision_id)
-    if source.repository.has_revision(prev_revid):
-        base_tree = source.repository.revision_tree(prev_revid)
-    else:
-        base_tree = target.repository.revision_tree(prev_revid)
-
-    builder = SvnCommitBuilder(target.repository, target, 
-                               [revision_id, prev_revid],
-                               target.get_config(),
-                               None,
-                               None,
-                               None,
-                               rev.properties, 
-                               None,
-                               base_tree.inventory)
-                         
-    builder.new_inventory = source.repository.get_inventory(revision_id)
-    replay_delta(builder, base_tree, old_tree)
-
-    try:
-        return builder.commit(rev.message)
-    except SubversionException, (_, num):
-        if num == svn.core.SVN_ERR_FS_TXN_OUT_OF_DATE:
-            raise DivergedBranches(source, target)
-        raise
-
 
 def push_new(target_repository, target_branch_path, source, 
-             stop_revision=None, validate=False):
+             stop_revision=None):
     """Push a revision into Subversion, creating a new branch.
 
     This will do a new commit in the target branch.
@@ -613,14 +605,12 @@ def push_new(target_repository, target_branch_path, source,
     :param target_branch_path: Path to create new branch at
     :param source: Branch to pull the revision from
     :param revision_id: Revision id of the revision to push
-    :param validate: Whether to check the committed revision matches 
-        the source revision.
     """
     assert isinstance(source, Branch)
     if stop_revision is None:
         stop_revision = source.last_revision()
     history = source.revision_history()
-    revhistory = deepcopy(history)
+    revhistory = list(history)
     start_revid = NULL_REVISION
     while len(revhistory) > 0:
         revid = revhistory.pop()
@@ -643,6 +633,11 @@ def push_new(target_repository, target_branch_path, source,
             """See Branch.get_config()."""
             return None
 
+        def revision_id_to_revno(self, revid):
+            if revid is None:
+                return 0
+            return history.index(revid)
+
         def last_revision_info(self):
             """See Branch.last_revision_info()."""
             last_revid = self.last_revision()
@@ -665,13 +660,12 @@ def push_new(target_repository, target_branch_path, source,
             """See SvnBranch.generate_revision_id()."""
             return self.repository.generate_revision_id(
                 revnum, self.get_branch_path(revnum), 
-                str(self.repository.get_scheme()))
+                self.repository.get_mapping())
 
-    push(ImaginaryBranch(target_repository), source, start_revid, 
-         validate=validate)
+    push(ImaginaryBranch(target_repository), source, start_revid)
 
 
-def push(target, source, revision_id, validate=False):
+def push(target, source, revision_id):
     """Push a revision into Subversion.
 
     This will do a new commit in the target branch.
@@ -679,41 +673,42 @@ def push(target, source, revision_id, validate=False):
     :param target: Branch to push to
     :param source: Branch to pull the revision from
     :param revision_id: Revision id of the revision to push
-    :param validate: Whether to check the committed revision matches 
-        the source revision.
     """
     assert isinstance(source, Branch)
-    mutter('pushing %r' % (revision_id))
     rev = source.repository.get_revision(revision_id)
-
-    base_revid = target.last_revision()
+    mutter('pushing %r (%r)' % (revision_id, rev.parent_ids))
 
     # revision on top of which to commit
-    assert (base_revid in rev.parent_ids or 
-            base_revid is None and rev.parent_ids == [])
+    if rev.parent_ids == []:
+        base_revid = None
+    else:
+        base_revid = rev.parent_ids[0]
 
-    old_tree = source.repository.revision_tree(revision_id)
-    base_tree = source.repository.revision_tree(base_revid)
+    source.lock_read()
+    try:
+        old_tree = source.repository.revision_tree(revision_id)
+        base_tree = source.repository.revision_tree(base_revid)
 
-    builder = SvnCommitBuilder(target.repository, target, 
-                               rev.parent_ids,
-                               target.get_config(),
-                               rev.timestamp,
-                               rev.timezone,
-                               rev.committer,
-                               rev.properties, 
-                               revision_id,
-                               base_tree.inventory)
-                         
-    builder.new_inventory = source.repository.get_inventory(revision_id)
-    replay_delta(builder, base_tree, old_tree)
+        builder = SvnCommitBuilder(target.repository, target, rev.parent_ids,
+                                   target.get_config(), rev.timestamp,
+                                   rev.timezone, rev.committer, rev.properties, 
+                                   revision_id, base_tree.inventory)
+                             
+        replay_delta(builder, base_tree, old_tree)
+    finally:
+        source.unlock()
     try:
         builder.commit(rev.message)
     except SubversionException, (_, num):
         if num == svn.core.SVN_ERR_FS_TXN_OUT_OF_DATE:
             raise DivergedBranches(source, target)
         raise
-    if validate:
+    except ChangesRootLHSHistory:
+        raise BzrError("Unable to push revision %r because it would change the ordering of existing revisions on the Subversion repository root. Use rebase and try again or push to a non-root path" % revision_id)
+
+    # FIXME: copy revisions signature
+
+    if 'validate' in debug.debug_flags:
         crev = target.repository.get_revision(revision_id)
         ctree = target.repository.revision_tree(revision_id)
         treedelta = ctree.changes_from(old_tree)
@@ -735,54 +730,54 @@ class InterToSvnRepository(InterRepository):
         """See InterRepository._get_repo_format_to_test()."""
         return None
 
-    def copy_content(self, revision_id=None, basis=None, pb=None):
+    def copy_content(self, revision_id=None, pb=None):
         """See InterRepository.copy_content."""
-        assert revision_id is not None, "fetching all revisions not supported"
-        # Go back over the LHS parent until we reach a revid we know
-        todo = []
-        while not self.target.has_revision(revision_id):
-            todo.append(revision_id)
-            revision_id = self.source.revision_parents(revision_id)[0]
-            if revision_id == NULL_REVISION:
-                raise UnrelatedBranches()
-        if todo == []:
-            # Nothing to do
-            return
-        mutter("pushing %r into svn" % todo)
-        target_branch = None
-        for revision_id in todo:
-            if pb is not None:
-                pb.update("pushing revisions", todo.index(revision_id), len(todo))
-            rev = self.source.get_revision(revision_id)
+        self.source.lock_read()
+        try:
+            assert revision_id is not None, "fetching all revisions not supported"
+            # Go back over the LHS parent until we reach a revid we know
+            todo = []
+            while not self.target.has_revision(revision_id):
+                todo.append(revision_id)
+                revision_id = self.source.revision_parents(revision_id)[0]
+                if revision_id == NULL_REVISION:
+                    raise UnrelatedBranches()
+            if todo == []:
+                # Nothing to do
+                return
+            mutter("pushing %r into svn" % todo)
+            target_branch = None
+            for revision_id in todo:
+                if pb is not None:
+                    pb.update("pushing revisions", todo.index(revision_id), len(todo))
+                rev = self.source.get_revision(revision_id)
 
-            mutter('pushing %r' % (revision_id))
+                mutter('pushing %r' % (revision_id))
 
-            old_tree = self.source.revision_tree(revision_id)
-            parent_revid = rev.parent_ids[0]
-            base_tree = self.source.revision_tree(parent_revid)
+                old_tree = self.source.revision_tree(revision_id)
+                parent_revid = rev.parent_ids[0]
+                base_tree = self.source.revision_tree(parent_revid)
 
-            (bp, _, _) = self.target.lookup_revision_id(parent_revid)
-            if target_branch is None:
-                target_branch = Branch.open(urlutils.join(self.target.base, bp))
-            if target_branch.get_branch_path() != bp:
-                target_branch.set_branch_path(bp)
+                (bp, _, _) = self.target.lookup_revision_id(parent_revid)
+                if target_branch is None:
+                    target_branch = Branch.open(urlutils.join(self.target.base, bp))
+                if target_branch.get_branch_path() != bp:
+                    target_branch.set_branch_path(bp)
 
-            builder = SvnCommitBuilder(self.target, target_branch, 
-                               rev.parent_ids,
-                               target_branch.get_config(),
-                               rev.timestamp,
-                               rev.timezone,
-                               rev.committer,
-                               rev.properties, 
-                               revision_id,
-                               base_tree.inventory)
-                         
-            builder.new_inventory = self.source.get_inventory(revision_id)
-            replay_delta(builder, base_tree, old_tree)
-            builder.commit(rev.message)
+                builder = SvnCommitBuilder(self.target, target_branch, 
+                                   rev.parent_ids, target_branch.get_config(),
+                                   rev.timestamp, rev.timezone, rev.committer,
+                                   rev.properties, revision_id, base_tree.inventory)
+                             
+                replay_delta(builder, base_tree, old_tree)
+                builder.commit(rev.message)
+
+                # FIXME: Copy revision signature for rev
+        finally:
+            self.source.unlock()
  
 
-    def fetch(self, revision_id=None, pb=None):
+    def fetch(self, revision_id=None, pb=None, find_ghosts=False):
         """Fetch revisions. """
         self.copy_content(revision_id=revision_id, pb=pb)
 
