@@ -15,42 +15,67 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Cache of the Subversion history log."""
 
-from bzrlib import urlutils
+from bzrlib import urlutils, debug
 from bzrlib.errors import NoSuchRevision
+from bzrlib.trace import mutter
 import bzrlib.ui as ui
 
 from svn.core import SubversionException, Pool
 from transport import SvnRaTransport
 import svn.core
 
-import base64
-
 from cache import sqlite3
 
-def _escape_commit_message(message):
-    """Replace xml-incompatible control characters."""
-    if message is None:
+LOG_CHUNK_LIMIT = 0
+
+def changes_find_prev_location(paths, branch_path, revnum):
+    assert isinstance(paths, dict)
+    assert isinstance(branch_path, str)
+    assert isinstance(revnum, int)
+    if revnum == 0:
+        assert branch_path == ""
         return None
-    import re
-    # FIXME: RBC 20060419 this should be done by the revision
-    # serialiser not by commit. Then we can also add an unescaper
-    # in the deserializer and start roundtripping revision messages
-    # precisely. See repository_implementations/test_repository.py
+    # If there are no special cases, just go try the 
+    # next revnum in history
+    revnum -= 1
+
+    # Make sure we get the right location for next time, if 
+    # the branch itself was copied
+    if (paths.has_key(branch_path) and 
+        paths[branch_path][0] in ('R', 'A')):
+        if paths[branch_path][1] is None: 
+            return None # Was added here
+        revnum = paths[branch_path][2]
+        branch_path = paths[branch_path][1].encode("utf-8")
+        return (branch_path, revnum)
     
-    # Python strings can include characters that can't be
-    # represented in well-formed XML; escape characters that
-    # aren't listed in the XML specification
-    # (http://www.w3.org/TR/REC-xml/#NT-Char).
-    message, _ = re.subn(
-        u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
-        lambda match: match.group(0).encode('unicode_escape'),
-        message)
-    return message
+    # Make sure we get the right location for the next time if 
+    # one of the parents changed
+
+    # Path names need to be sorted so the longer paths 
+    # override the shorter ones
+    for p in sorted(paths.keys(), reverse=True):
+        if paths[p][0] == 'M':
+            continue
+        if branch_path.startswith(p+"/"):
+            assert paths[p][0] in ('A', 'R'), "Parent %r wasn't added" % p
+            assert paths[p][1] is not None, \
+                "Empty parent %r added, but child %r wasn't added !?" % (p, branch_path)
+
+            revnum = paths[p][2]
+            branch_path = paths[p][1].encode("utf-8") + branch_path[len(p):]
+            return (branch_path, revnum)
+
+    return (branch_path, revnum)
+
+
+
+
 
 
 class LogWalker(object):
     """Easy way to access the history of a Subversion repository."""
-    def __init__(self, transport=None, cache_db=None):
+    def __init__(self, transport, cache_db=None, limit=None):
         """Create a new instance.
 
         :param transport:   SvnRaTransport to use to access the repository.
@@ -59,7 +84,13 @@ class LogWalker(object):
         """
         assert isinstance(transport, SvnRaTransport)
 
-        self.transport = SvnRaTransport(transport.base)
+        self.url = transport.base
+        self._transport = None
+
+        if limit is not None:
+            self._limit = limit
+        else:
+            self._limit = LOG_CHUNK_LIMIT
 
         if cache_db is None:
             self.db = sqlite3.connect(":memory:")
@@ -67,58 +98,67 @@ class LogWalker(object):
             self.db = cache_db
 
         self.db.executescript("""
-          create table if not exists revision(revno integer unique, author text, message text, date text);
-          create unique index if not exists revision_revno on revision (revno);
           create table if not exists changed_path(rev integer, action text, path text, copyfrom_path text, copyfrom_rev integer);
           create index if not exists path_rev on changed_path(rev);
           create unique index if not exists path_rev_path on changed_path(rev, path);
           create unique index if not exists path_rev_path_action on changed_path(rev, path, action);
         """)
         self.db.commit()
-        self.saved_revnum = self.db.execute("SELECT MAX(revno) FROM revision").fetchone()[0]
+        self.saved_revnum = self.db.execute("SELECT MAX(rev) FROM changed_path").fetchone()[0]
         if self.saved_revnum is None:
             self.saved_revnum = 0
 
+    def _get_transport(self):
+        if self._transport is not None:
+            return self._transport
+        self._transport = SvnRaTransport(self.url)
+        return self._transport
+
+    def mutter(self, text):
+        if "cache" in debug.debug_flags:
+            mutter(text)
+
     def fetch_revisions(self, to_revnum=None):
         """Fetch information about all revisions in the remote repository
         until to_revnum.
 
         :param to_revnum: End of range to fetch information for
         """
-        if to_revnum is None:
-            to_revnum = self.transport.get_latest_revnum()
-        else:
-            to_revnum = max(self.transport.get_latest_revnum(), to_revnum)
+        if to_revnum <= self.saved_revnum:
+            return
+        latest_revnum = self._get_transport().get_latest_revnum()
+        to_revnum = max(latest_revnum, to_revnum)
 
         pb = ui.ui_factory.nested_progress_bar()
 
-        def rcvr(orig_paths, rev, author, date, message, pool):
-            pb.update('fetching svn revision info', rev, to_revnum)
+        def rcvr(log_entry, pool):
+            pb.update('fetching svn revision info', log_entry.revision, to_revnum)
+            orig_paths = log_entry.changed_paths
             if orig_paths is None:
                 orig_paths = {}
             for p in orig_paths:
                 copyfrom_path = orig_paths[p].copyfrom_path
-                if copyfrom_path:
+                if copyfrom_path is not None:
                     copyfrom_path = copyfrom_path.strip("/")
 
                 self.db.execute(
                      "replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev) values (?, ?, ?, ?, ?)", 
-                     (rev, p.strip("/"), orig_paths[p].action, copyfrom_path, orig_paths[p].copyfrom_rev))
+                     (log_entry.revision, p.strip("/"), orig_paths[p].action, copyfrom_path, orig_paths[p].copyfrom_rev))
+                # Work around nasty memory leak in Subversion
+                orig_paths[p]._parent_pool.destroy()
 
-            if message is not None:
-                message = base64.b64encode(message)
-
-            self.db.execute("replace into revision (revno, author, date, message) values (?,?,?,?)", (rev, author, date, message))
-
-            self.saved_revnum = rev
+            self.saved_revnum = log_entry.revision
             if self.saved_revnum % 1000 == 0:
                 self.db.commit()
 
-        pool = Pool()
         try:
             try:
-                self.transport.get_log("/", self.saved_revnum, to_revnum, 
-                               0, True, True, rcvr, pool)
+                while self.saved_revnum < to_revnum:
+                    pool = Pool()
+                    self._get_transport().get_log("", self.saved_revnum, 
+                                             to_revnum, self._limit, True, 
+                                             True, [], rcvr, pool)
+                    pool.destroy()
             finally:
                 pb.finished()
         except SubversionException, (_, num):
@@ -127,101 +167,95 @@ class LogWalker(object):
                     revision="Revision number %d" % to_revnum)
             raise
         self.db.commit()
-        pool.destroy()
 
-    def follow_path(self, path, revnum):
-        """Return iterator over all the revisions between revnum and 
-        0 named path or inside path.
+    def iter_changes(self, path, revnum):
+        """Return iterator over all the revisions between revnum and 0 named path or inside path.
 
-        :param path:   Branch path to start reporting (in revnum)
-        :param revnum:        Start revision.
-
-        :return: An iterators that yields tuples with (path, paths, revnum)
-        where paths is a dictionary with all changes that happened in path 
-        in revnum.
+        :param path:    Branch path to start reporting (in revnum)
+        :param revnum:  Start revision.
+        :return: An iterator that yields tuples with (path, paths, revnum)
+            where paths is a dictionary with all changes that happened in path 
+            in revnum.
         """
         assert revnum >= 0
 
         if revnum == 0 and path == "":
             return
 
+        self.mutter("follow path %r:%r" % (path, revnum))
+
+        recurse = (path != "")
+
         path = path.strip("/")
 
         while revnum >= 0:
-            revpaths = self.get_revision_paths(revnum, path)
+            assert revnum > 0 or path == ""
+            revpaths = self.get_revision_paths(revnum, path, recurse=recurse)
+
+            next = changes_find_prev_location(revpaths, path, revnum)
 
             if revpaths != {}:
                 yield (path, revpaths, revnum)
 
-            if revpaths.has_key(path):
-                if revpaths[path][1] is None:
-                    if revpaths[path][0] in ('A', 'R'):
-                        # this path didn't exist before this revision
-                        return
-                else:
-                    # In this revision, this path was copied from 
-                    # somewhere else
-                    revnum = revpaths[path][2]
-                    path = revpaths[path][1]
-                    continue
-            revnum -= 1
+            if next is None:
+                break
 
-    def get_revision_paths(self, revnum, path=None):
+            (path, revnum) = next
+
+    def get_revision_paths(self, revnum, path=None, recurse=False):
         """Obtain dictionary with all the changes in a particular revision.
 
         :param revnum: Subversion revision number
         :param path: optional path under which to return all entries
+        :param recurse: Report changes to parents as well
         :returns: dictionary with paths as keys and 
                   (action, copyfrom_path, copyfrom_rev) as values.
         """
 
         if revnum == 0:
+            assert path is None or path == ""
             return {'': ('A', None, -1)}
+
+        self.mutter("revision paths: %r" % revnum)
                 
-        if revnum > self.saved_revnum:
-            self.fetch_revisions(revnum)
+        self.fetch_revisions(revnum)
 
         query = "select path, action, copyfrom_path, copyfrom_rev from changed_path where rev="+str(revnum)
         if path is not None and path != "":
-            query += " and (path='%s' or path like '%s/%%')" % (path, path)
+            query += " and (path='%s' or path like '%s/%%'" % (path, path)
+            if recurse:
+                query += " or ('%s' LIKE path || '/%%')" % path
+            query += ")"
 
         paths = {}
         for p, act, cf, cr in self.db.execute(query):
+            if cf is not None:
+                cf = cf.encode("utf-8")
             paths[p.encode("utf-8")] = (act, cf, cr)
         return paths
 
-    def get_revision_info(self, revnum):
-        """Obtain basic information for a specific revision.
-
-        :param revnum: Revision number.
-        :returns: Tuple with author, log message and date of the revision.
-        """
-        assert revnum >= 0
-        if revnum == 0:
-            return (None, None, None)
-        if revnum > self.saved_revnum:
-            self.fetch_revisions(revnum)
-        (author, message, date) = self.db.execute("select author, message, date from revision where revno="+ str(revnum)).fetchone()
-        if message is not None:
-            message = _escape_commit_message(base64.b64decode(message))
-        return (author, message, date)
-
-    def find_latest_change(self, path, revnum, recurse=False):
+    def find_latest_change(self, path, revnum, include_parents=False,
+                           include_children=False):
         """Find latest revision that touched path.
 
         :param path: Path to check for changes
         :param revnum: First revision to check
         """
-        assert isinstance(path, basestring)
+        assert isinstance(path, str)
         assert isinstance(revnum, int) and revnum >= 0
-        if revnum > self.saved_revnum:
-            self.fetch_revisions(revnum)
+        self.fetch_revisions(revnum)
 
-        if recurse:
-            extra = " or path like '%s/%%'" % path.strip("/")
-        else:
-            extra = ""
-        query = "select rev from changed_path where (path='%s' or ('%s' like (path || '/%%') and (action = 'R' or action = 'A'))%s) and rev <= %d order by rev desc limit 1" % (path.strip("/"), path.strip("/"), extra, revnum)
+        self.mutter("latest change: %r:%r" % (path, revnum))
+
+        extra = ""
+        if include_children:
+            if path == "":
+                extra += " OR path LIKE '%'"
+            else:
+                extra += " OR path LIKE '%s/%%'" % path.strip("/")
+        if include_parents:
+            extra += " OR ('%s' LIKE (path || '/%%') AND (action = 'R' OR action = 'A'))" % path.strip("/")
+        query = "SELECT rev FROM changed_path WHERE (path='%s'%s) AND rev <= %d ORDER BY rev DESC LIMIT 1" % (path.strip("/"), extra, revnum)
 
         row = self.db.execute(query).fetchone()
         if row is None and path == "":
@@ -238,16 +272,22 @@ class LogWalker(object):
         :param path:  Path to check
         :param revnum:  Revision to check
         """
-        if revnum > self.saved_revnum:
-            self.fetch_revisions(revnum)
+        self.fetch_revisions(revnum)
+        self.mutter("touches path %r:%r" % (path, revnum))
         if revnum == 0:
             return (path == "")
         return (self.db.execute("select 1 from changed_path where path='%s' and rev=%d" % (path, revnum)).fetchone() is not None)
 
     def find_children(self, path, revnum):
-        """Find all children of path in revnum."""
+        """Find all children of path in revnum.
+
+        :param path:  Path to check
+        :param revnum:  Revision to check
+        """
+        self.mutter("find children %r:%r" % (path, revnum))
         path = path.strip("/")
-        ft = self.transport.check_path(path, revnum)
+        transport = self._get_transport()
+        ft = transport.check_path(path, revnum)
         if ft == svn.core.svn_node_file:
             return []
         assert ft == svn.core.svn_node_dir
@@ -296,17 +336,15 @@ class LogWalker(object):
                 pass
         pool = Pool()
         editor = TreeLister(path)
-        edit, baton = svn.delta.make_editor(editor, pool)
-        old_base = self.transport.base
+        old_base = transport.base
         try:
-            root_repos = self.transport.get_repos_root()
-            self.transport.reparent(urlutils.join(root_repos, path))
-            reporter = self.transport.do_update(
-                            revnum,  True, edit, baton, pool)
+            root_repos = transport.get_svn_repos_root()
+            transport.reparent(urlutils.join(root_repos, path))
+            reporter = transport.do_update(revnum, True, editor, pool)
             reporter.set_path("", revnum, True, None, pool)
             reporter.finish_report(pool)
         finally:
-            self.transport.reparent(old_base)
+            transport.reparent(old_base)
         return editor.files
 
     def get_previous(self, path, revnum):
@@ -316,8 +354,8 @@ class LogWalker(object):
         :param revnum:  Revision to check
         """
         assert revnum >= 0
-        if revnum > self.saved_revnum:
-            self.fetch_revisions(revnum)
+        self.fetch_revisions(revnum)
+        self.mutter("get previous %r:%r" % (path, revnum))
         if revnum == 0:
             return (None, -1)
         row = self.db.execute("select action, copyfrom_path, copyfrom_rev from changed_path where path='%s' and rev=%d" % (path, revnum)).fetchone()
