@@ -16,12 +16,14 @@
 
 """Checkout tests."""
 
+from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoRepositoryPresent
+from bzrlib.errors import NoRepositoryPresent, UninitializableFormat
 from bzrlib.tests import TestCase
 
 from convert import SvnConverter
-from workingtree import SvnWorkingTreeFormat, SvnWorkingTreeDirFormat
+from bzrlib.plugins.svn.workingtree import SvnWorkingTreeFormat
+from bzrlib.plugins.svn.format import SvnWorkingTreeDirFormat
 from tests import TestCaseWithSubversionRepository
 
 class TestWorkingTreeFormat(TestCase):
@@ -45,7 +47,11 @@ class TestCheckoutFormat(TestCase):
         self.format = SvnWorkingTreeDirFormat()
 
     def test_get_converter(self):
-        self.assertIsInstance(self.format.get_converter(), SvnConverter)
+        self.assertRaises(NotImplementedError, self.format.get_converter)
+
+    def test_initialize(self):
+        self.assertRaises(UninitializableFormat, 
+                          self.format.initialize_on_transport, None)
 
 
 class TestCheckout(TestCaseWithSubversionRepository):
@@ -62,5 +68,26 @@ class TestCheckout(TestCaseWithSubversionRepository):
     def test_find_repository(self):
         self.make_client("d", "dc")
         x = self.open_checkout_bzrdir("dc")
-        self.assertTrue(hasattr(x.find_repository(), "uuid"))
+        self.assertRaises(NoRepositoryPresent, x.find_repository)
+
+    def test__find_repository(self):
+        self.make_client("d", "dc")
+        x = self.open_checkout_bzrdir("dc")
+        self.assertTrue(hasattr(x._find_repository(), "uuid"))
+
+    def test_needs_format_conversion_default(self):
+        self.make_client("d", "dc")
+        x = self.open_checkout_bzrdir("dc")
+        self.assertTrue(x.needs_format_conversion())
+
+    def test_needs_format_conversion_self(self):
+        self.make_client("d", "dc")
+        x = self.open_checkout_bzrdir("dc")
+        self.assertFalse(x.needs_format_conversion(SvnWorkingTreeDirFormat()))
+        
+    def test_checkout_checkout(self):
+        """Test making a checkout of a checkout."""
+        self.make_client("d", "dc")
+        x = Branch.open("dc")
+        x.create_checkout("de", lightweight=True)
 
