@@ -21,7 +21,9 @@ from bzrlib.errors import NotBranchError, BzrError
 from bzrlib.tests import TestCase
 from scheme import (ListBranchingScheme, NoBranchingScheme, 
                     BranchingScheme, TrunkBranchingScheme, 
-                    SingleBranchingScheme, UnknownBranchingScheme,
+                    SingleBranchingSchemev0, 
+                    SingleBranchingScheme, 
+                    UnknownBranchingScheme,
                     parse_list_scheme_text, find_commit_paths, 
                     guess_scheme_from_branch_path, guess_scheme_from_history,
                     guess_scheme_from_path, scheme_from_branch_list)
@@ -73,6 +75,11 @@ class BranchingSchemeTest(TestCase):
 
     def test_find_scheme_single(self):
         scheme = BranchingScheme.find_scheme("single-habla")
+        self.assertIsInstance(scheme, SingleBranchingSchemev0)
+        self.assertEqual("habla", scheme.path)
+
+    def test_find_scheme_single1(self):
+        scheme = BranchingScheme.find_scheme("single1-aGFibGE.")
         self.assertIsInstance(scheme, SingleBranchingScheme)
         self.assertEqual("habla", scheme.path)
 
@@ -138,6 +145,10 @@ class ListScheme(TestCase):
     def setUp(self):
         self.scheme = ListBranchingScheme(["foo", "bar/bloe"])
 
+    def test_create_from_string(self):
+        self.scheme = ListBranchingScheme('QlpoOTFBWSZTWXb2s-UAAADBAAAQAQCgACGYGYQYXckU4UJB29rPlA..')
+        self.assertEquals(["foo"], self.scheme.branch_list)
+
     def test_is_tag_empty(self):
         self.assertFalse(self.scheme.is_tag(""))
 
@@ -172,6 +183,18 @@ class ListScheme(TestCase):
         self.assertFalse(scheme.is_branch("blie/trunk/bloe/bla"))
         self.assertFalse(scheme.is_branch("bla"))
 
+    def test_is_branch_parent_root_root(self):
+        self.assertFalse(ListBranchingScheme([""]).is_branch_parent(""))
+
+    def test_is_branch_parent_root(self):
+        self.assertTrue(ListBranchingScheme(["trunk/*"]).is_branch_parent("trunk"))
+
+    def test_is_branch_parent_other(self):
+        self.assertFalse(ListBranchingScheme(["trunk/*"]).is_branch_parent("trunk/foo"))
+
+    def test_is_tag_parent_other(self):
+        self.assertFalse(ListBranchingScheme(["trunk"]).is_tag_parent("trunk/foo"))
+
     def test_is_branch_slashsub(self):
         self.assertTrue(self.scheme.is_branch("/foo"))
 
@@ -240,7 +263,7 @@ class ListScheme(TestCase):
         self.assertEqual(self.scheme.unprefix("bar/bloe"), ("bar/bloe", ""))
 
     def test_str(self):
-        self.assertEqual("list-QlpoOTFBWSZTWSDz6woAAAPRgAAQAACzBJAAIAAiDRo9QgyYjmbjatAeLuSKcKEgQefWFA==", str(self.scheme))
+        self.assertEqual("list-QlpoOTFBWSZTWSDz6woAAAPRgAAQAACzBJAAIAAiDRo9QgyYjmbjatAeLuSKcKEgQefWFA..", str(self.scheme))
 
     def test_parse_text(self):
         self.assertEqual(["bla/bloe"], parse_list_scheme_text("bla/bloe\n"))
@@ -465,7 +488,13 @@ class SingleBranchingSchemeTests(TestCase):
         self.assertRaises(BzrError, SingleBranchingScheme, "")
 
     def test_str(self):
-        self.assertEquals("single-ha/bla", str(SingleBranchingScheme("ha/bla")))
+        self.assertEquals("single1-aGEvYmxh", str(SingleBranchingScheme("ha/bla")))
+
+    def test_str_no_need(self):
+        self.assertEquals("single-ha", str(SingleBranchingScheme("ha")))
+        
+    def test_str_v0(self):
+        self.assertEquals("single-ha/bla", str(SingleBranchingSchemev0("ha/bla")))
 
 
 class FindCommitPathsTester(TestCase):
