@@ -17,7 +17,8 @@
 """Subversion transport tests."""
 
 from tests import TestCaseWithSubversionRepository
-from bzrlib.errors import NotBranchError, NoSuchFile, FileExists
+from bzrlib.errors import NotBranchError, NoSuchFile, FileExists, InvalidURL
+from bzrlib import urlutils
 from transport import SvnRaTransport, bzr_to_svn_url
 from unittest import TestCase
 
@@ -25,7 +26,7 @@ import os
 
 class SvnRaTest(TestCaseWithSubversionRepository):
     def test_open_nonexisting(self):
-        self.assertRaises(NotBranchError, SvnRaTransport, 
+        self.assertRaises(InvalidURL, SvnRaTransport, 
                           "svn+nonexisting://foo/bar")
 
     def test_create(self):
@@ -115,7 +116,7 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         t = SvnRaTransport(repos_url)
         tt = t.clone()
         self.assertEqual(tt.base, t.base)
-        tt.reparent(os.path.join(t.base, "dir"))
+        tt.reparent(urlutils.join(t.base, "dir"))
         self.assertNotEqual(tt.base, t.base)
 
     def test_mkdir(self):
@@ -167,13 +168,13 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         self.client_commit("dc", "Bla")
 
         t = SvnRaTransport("%s/dir" % repos_url)
-        root = t.get_repos_root()
+        root = t.get_svn_repos_root()
         self.assertEqual(repos_url, root)
 
     def test_local_abspath(self):
         repos_url = self.make_client('d', 'dc')
         t = SvnRaTransport("%s" % repos_url)
-        self.assertEquals(os.path.join(self.test_dir, "d"), t.local_abspath('.'))
+        self.assertEquals(urlutils.join(self.test_dir, "d"), t.local_abspath('.'))
  
 
 class UrlConversionTest(TestCase):
