@@ -22,14 +22,14 @@ from bzrlib.repository import Repository
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 
 from errors import RebaseNotPresent
-from fileids import generate_svn_file_id
 from format import get_rich_root_format
-from repository import MAPPING_VERSION
+from mapping import (BzrSvnMappingv3FileProps, BzrSvnMappingv2, 
+                     BzrSvnMappingv1)
+from scheme import TrunkBranchingScheme
 from tests import TestCaseWithSubversionRepository
 from upgrade import (upgrade_repository, upgrade_branch,
                      upgrade_workingtree, UpgradeChangesContent, 
-                     parse_legacy_revision_id, create_upgraded_revid, 
-                     generate_upgrade_map)
+                     create_upgraded_revid, generate_upgrade_map)
 
 class TestUpgradeChangesContent(TestCase):
     def test_init(self):
@@ -38,37 +38,13 @@ class TestUpgradeChangesContent(TestCase):
 
 
 class ParserTests(TestCase):
-    def test_current(self):
-        self.assertEqual(("uuid", "trunk", 1, "trunk0", 3), 
-                parse_legacy_revision_id("svn-v3-trunk0:uuid:trunk:1"))
-
-    def test_current_undefined(self):
-        self.assertEqual(("uuid", "trunk", 1, None, 3), 
-                parse_legacy_revision_id("svn-v3-undefined:uuid:trunk:1"))
-
-    def test_legacy2(self):
-        self.assertEqual(("uuid", "trunk", 1, None, 2), 
-                         parse_legacy_revision_id("svn-v2:1@uuid-trunk"))
-
-    def test_legacy(self):
-        self.assertEqual(("uuid", "trunk", 1, None, 1), 
-                         parse_legacy_revision_id("svn-v1:1@uuid-trunk"))
-
-    def test_except(self):
-        self.assertRaises(InvalidRevisionId, 
-                         parse_legacy_revision_id, "svn-v0:1@uuid-trunk")
-
-    def test_except_nonsvn(self):
-        self.assertRaises(InvalidRevisionId, 
-                         parse_legacy_revision_id, "blah")
-
     def test_create_upgraded_revid_new(self):
-        self.assertEqual("bla-svn%d-upgrade" % MAPPING_VERSION,
-                         create_upgraded_revid("bla"))
+        self.assertEqual("bla-svn3-upgrade",
+                         create_upgraded_revid("bla", "-svn3"))
 
     def test_create_upgraded_revid_upgrade(self):
-        self.assertEqual("bla-svn%d-upgrade" % MAPPING_VERSION,
-                         create_upgraded_revid("bla-svn1-upgrade"))
+        self.assertEqual("bla-svn3-upgrade",
+                         create_upgraded_revid("bla-svn1-upgrade", "-svn3"))
 
 
 def skip_no_rebase(unbound):
@@ -91,7 +67,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
         dir.create_branch()
@@ -104,7 +80,8 @@ class UpgradeTests(TestCaseWithSubversionRepository):
 
         upgrade_repository(newrepos, oldrepos, allow_changes=True)
 
-        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", "none")))
+        mapping = oldrepos.get_mapping()
+        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", mapping)))
 
     @skip_no_rebase
     def test_single_custom(self):
@@ -114,7 +91,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
         dir.create_branch()
@@ -127,10 +104,13 @@ class UpgradeTests(TestCaseWithSubversionRepository):
 
         upgrade_repository(newrepos, oldrepos, allow_changes=True)
 
-        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", "none")))
-        self.assertTrue(newrepos.has_revision("customrev-svn%d-upgrade" % MAPPING_VERSION))
-        self.assertTrue([oldrepos.generate_revision_id(1, "", "none")],
-                        newrepos.revision_parents("customrev-svn%d-upgrade" % MAPPING_VERSION))
+        mapping = oldrepos.get_mapping()
+        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", mapping)))
+        self.assertTrue(newrepos.has_revision("customrev%s-upgrade" % mapping.upgrade_suffix))
+        newrepos.lock_read()
+        self.assertTrue((oldrepos.generate_revision_id(1, "", mapping),),
+                        tuple(newrepos.revision_parents("customrev%s-upgrade" % mapping.upgrade_suffix)))
+        newrepos.unlock()
 
     @skip_no_rebase
     def test_single_keep_parent_fileid(self):
@@ -154,10 +134,10 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         wt.commit(message='fix moredata', rev_id="customrev")
 
         upgrade_repository(newrepos, oldrepos, allow_changes=True)
-
-        tree = newrepos.revision_tree("customrev-svn%d-upgrade" % MAPPING_VERSION)
+        mapping = oldrepos.get_mapping()
+        tree = newrepos.revision_tree("customrev%s-upgrade" % mapping.upgrade_suffix)
         self.assertEqual("specificid", tree.inventory.path2id("a"))
-        self.assertEqual(generate_svn_file_id(oldrepos.uuid, 1, "", "a"), 
+        self.assertEqual(mapping.generate_file_id(oldrepos.uuid, 1, "", u"a"), 
                          tree.inventory.path2id("b"))
 
     @skip_no_rebase
@@ -169,7 +149,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
         dir.create_branch()
@@ -185,16 +165,25 @@ class UpgradeTests(TestCaseWithSubversionRepository):
 
         tree = newrepos.revision_tree("svn-v1:1@%s-" % oldrepos.uuid)
 
+        newrepos.lock_write()
+        newrepos.start_write_group()
+
+        mapping = oldrepos.get_mapping()
         vf = newrepos.weave_store.get_weave_or_empty(tree.inventory.path2id("a"), newrepos.get_transaction())
-        vf.clone_text("customrev-svn%d-upgrade" % MAPPING_VERSION,
+        vf.clone_text("customrev%s-upgrade" % mapping.upgrade_suffix,
                 "svn-v1:1@%s-" % oldrepos.uuid, ["svn-v1:1@%s-" % oldrepos.uuid])
 
+        newrepos.commit_write_group()
+        newrepos.unlock()
+
         upgrade_repository(newrepos, oldrepos, allow_changes=True)
 
-        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", "none")))
-        self.assertTrue(newrepos.has_revision("customrev-svn%d-upgrade" % MAPPING_VERSION))
-        self.assertTrue([oldrepos.generate_revision_id(1, "", "none")],
-                        newrepos.revision_parents("customrev-svn%d-upgrade" % MAPPING_VERSION))
+        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", mapping)))
+        self.assertTrue(newrepos.has_revision("customrev%s-upgrade" % mapping.upgrade_suffix))
+        newrepos.lock_read()
+        self.assertTrue((oldrepos.generate_revision_id(1, "", mapping),),
+                        tuple(newrepos.revision_parents("customrev%s-upgrade" % mapping.upgrade_suffix)))
+        newrepos.unlock()
 
     @skip_no_rebase
     def test_more_custom(self):
@@ -204,7 +193,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         dir.create_branch()
         wt = dir.create_workingtree()
@@ -216,20 +205,23 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         file("f/a", 'w').write("blackfield")
         wt.commit(message='fix it again', rev_id="anotherrev")
 
+        mapping = oldrepos.get_mapping()
         renames = upgrade_repository(newrepos, oldrepos, allow_changes=True)
         self.assertEqual({
             'svn-v1:1@%s-' % oldrepos.uuid: 'svn-v3-none:%s::1' % oldrepos.uuid,
-            "customrev": "customrev-svn%d-upgrade" % MAPPING_VERSION,
-            "anotherrev": "anotherrev-svn%d-upgrade" % MAPPING_VERSION},
+            "customrev": "customrev%s-upgrade" % mapping.upgrade_suffix,
+            "anotherrev": "anotherrev%s-upgrade" % mapping.upgrade_suffix},
             renames)
 
-        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", "none")))
-        self.assertTrue(newrepos.has_revision("customrev-svn%d-upgrade" % MAPPING_VERSION))
-        self.assertTrue(newrepos.has_revision("anotherrev-svn%d-upgrade" % MAPPING_VERSION))
-        self.assertTrue([oldrepos.generate_revision_id(1, "", "none")],
-                        newrepos.revision_parents("customrev-svn%d-upgrade" % MAPPING_VERSION))
-        self.assertTrue(["customrev-svn%d-upgrade" % MAPPING_VERSION],
-                        newrepos.revision_parents("anotherrev-svn%d-upgrade" % MAPPING_VERSION))
+        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(1, "", mapping)))
+        self.assertTrue(newrepos.has_revision("customrev%s-upgrade" % mapping.upgrade_suffix))
+        self.assertTrue(newrepos.has_revision("anotherrev%s-upgrade" % mapping.upgrade_suffix))
+        newrepos.lock_read()
+        self.assertTrue((oldrepos.generate_revision_id(1, "", mapping),),
+                        tuple(newrepos.revision_parents("customrev%s-upgrade" % mapping.upgrade_suffix)))
+        self.assertTrue(("customrev-%s-upgrade" % mapping.upgrade_suffix,),
+                        tuple(newrepos.revision_parents("anotherrev%s-upgrade" % mapping.upgrade_suffix)))
+        newrepos.unlock()
 
     @skip_no_rebase
     def test_more_custom_branch(self):
@@ -239,7 +231,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         b = dir.create_branch()
         wt = dir.create_workingtree()
@@ -252,10 +244,11 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         wt.commit(message='fix it again', rev_id="anotherrev")
 
         upgrade_branch(b, oldrepos, allow_changes=True)
-        self.assertEqual([oldrepos.generate_revision_id(0, "", "none"),
-                          oldrepos.generate_revision_id(1, "", "none"),
-                          "customrev-svn%d-upgrade" % MAPPING_VERSION,
-                          "anotherrev-svn%d-upgrade" % MAPPING_VERSION
+        mapping = oldrepos.get_mapping()
+        self.assertEqual([oldrepos.generate_revision_id(0, "", mapping),
+                          oldrepos.generate_revision_id(1, "", mapping),
+                          "customrev%s-upgrade" % mapping.upgrade_suffix,
+                          "anotherrev%s-upgrade" % mapping.upgrade_suffix
                           ], b.revision_history())
 
     @skip_no_rebase
@@ -266,7 +259,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         newrepos = dir.create_repository()
         b = dir.create_branch()
         wt = dir.create_workingtree()
@@ -278,12 +271,13 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         file("f/a", 'w').write("blackfield")
         wt.commit(message='fix it again', rev_id="anotherrev")
 
+        mapping = oldrepos.get_mapping()
         upgrade_workingtree(wt, oldrepos, allow_changes=True)
         self.assertEquals(wt.last_revision(), b.last_revision())
-        self.assertEqual([oldrepos.generate_revision_id(0, "", "none"),
-                          oldrepos.generate_revision_id(1, "", "none"),
-                          "customrev-svn%d-upgrade" % MAPPING_VERSION,
-                          "anotherrev-svn%d-upgrade" % MAPPING_VERSION
+        self.assertEqual([oldrepos.generate_revision_id(0, "", mapping),
+                          oldrepos.generate_revision_id(1, "", mapping),
+                          "customrev%s-upgrade" % mapping.upgrade_suffix,
+                          "anotherrev%s-upgrade" % mapping.upgrade_suffix
                           ], b.revision_history())
 
     @skip_no_rebase
@@ -294,7 +288,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         dir.create_repository()
         b = dir.create_branch()
         wt = dir.create_workingtree()
@@ -318,7 +312,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "data")
 
         oldrepos = Repository.open(repos_url)
-        dir = BzrDir.create("f",format=get_rich_root_format())
+        dir = BzrDir.create("f", format=get_rich_root_format())
         dir.create_repository()
         b = dir.create_branch()
         wt = dir.create_workingtree()
@@ -331,7 +325,7 @@ class UpgradeTests(TestCaseWithSubversionRepository):
 
 class TestGenerateUpdateMapTests(TestCase):
     def test_nothing(self):
-        self.assertEquals({}, generate_upgrade_map(["bla", "bloe"]))
+        self.assertEquals({}, generate_upgrade_map(BzrSvnMappingv3FileProps(TrunkBranchingScheme()), ["bla", "bloe"]))
 
-    def test_v2(self):
-        self.assertEquals({"svn-v2:12@65390229-12b7-0310-b90b-f21a5aa7ec8e-trunk": "svn-v3-trunk0:65390229-12b7-0310-b90b-f21a5aa7ec8e:trunk:12"}, generate_upgrade_map(["svn-v2:12@65390229-12b7-0310-b90b-f21a5aa7ec8e-trunk", "bloe", "blaaa"]))
+    def test_v2_to_v3(self):
+        self.assertEquals({"svn-v2:12@65390229-12b7-0310-b90b-f21a5aa7ec8e-trunk": "svn-v3-trunk0:65390229-12b7-0310-b90b-f21a5aa7ec8e:trunk:12"}, generate_upgrade_map(BzrSvnMappingv3FileProps(TrunkBranchingScheme()), ["svn-v2:12@65390229-12b7-0310-b90b-f21a5aa7ec8e-trunk", "bloe", "blaaa"]))
