@@ -17,7 +17,7 @@
 
 from bzrlib import debug, urlutils
 from bzrlib.errors import (NoSuchFile, NotBranchError, TransportNotPossible, 
-                           FileExists, NotLocalUrl)
+                           FileExists, NotLocalUrl, InvalidURL)
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport
 
@@ -30,21 +30,49 @@ from errors import convert_svn_error, NoSvnRepositoryPresent
 
 svn_config = svn.core.svn_config_get_config(None)
 
+def get_client_string():
+    """Return a string that can be send as part of the User Agent string."""
+    return "bzr%s+bzr-svn%s" % (bzrlib.__version__, bzrlib.plugins.svn.__version__)
+
 
 def _create_auth_baton(pool):
     """Create a Subversion authentication baton. """
     # Give the client context baton a suite of authentication
     # providers.h
-    providers = [
+    providers = []
+
+    if svn.core.SVN_VER_MAJOR == 1 and svn.core.SVN_VER_MINOR >= 5:
+        import auth
+        providers += auth.SubversionAuthenticationConfig().get_svn_auth_providers()
+        providers += [auth.get_ssl_client_cert_pw_provider(1)]
+
+    providers += [
         svn.client.get_simple_provider(pool),
         svn.client.get_username_provider(pool),
         svn.client.get_ssl_client_cert_file_provider(pool),
         svn.client.get_ssl_client_cert_pw_file_provider(pool),
         svn.client.get_ssl_server_trust_file_provider(pool),
         ]
+
+    if hasattr(svn.client, 'get_windows_simple_provider'):
+        providers.append(svn.client.get_windows_simple_provider(pool))
+
+    if hasattr(svn.client, 'get_keychain_simple_provider'):
+        providers.append(svn.client.get_keychain_simple_provider(pool))
+
+    if hasattr(svn.client, 'get_windows_ssl_server_trust_provider'):
+        providers.append(svn.client.get_windows_ssl_server_trust_provider(pool))
+
     return svn.core.svn_auth_open(providers, pool)
 
 
+def create_svn_client(pool):
+    client = svn.client.create_context(pool)
+    client.auth_baton = _create_auth_baton(pool)
+    client.config = svn_config
+    return client
+
+
 # Don't run any tests on SvnTransport as it is not intended to be 
 # a full implementation of Transport
 def get_test_permutations():
@@ -73,12 +101,28 @@ def bzr_to_svn_url(url):
     return url.rstrip('/')
 
 
+def needs_busy(unbound):
+    """Decorator that marks a transport as busy before running a methd on it.
+    """
+    def convert(self, *args, **kwargs):
+        self._mark_busy()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self._unmark_busy()
+
+    convert.__doc__ = unbound.__doc__
+    convert.__name__ = unbound.__name__
+    return convert
+
+
 class Editor:
     """Simple object wrapper around the Subversion delta editor interface."""
-    def __init__(self, (editor, editor_baton)):
+    def __init__(self, transport, (editor, editor_baton)):
         self.editor = editor
         self.editor_baton = editor_baton
         self.recent_baton = []
+        self._transport = transport
 
     @convert_svn_error
     def open_root(self, base_revnum):
@@ -98,6 +142,7 @@ class Editor:
     def close(self):
         assert self.recent_baton == []
         svn.delta.editor_invoke_close_edit(self.editor, self.editor_baton)
+        self._transport._unmark_busy()
 
     @convert_svn_error
     def apply_textdelta(self, baton, *args, **kwargs):
@@ -106,10 +151,10 @@ class Editor:
                 *args, **kwargs)
 
     @convert_svn_error
-    def change_dir_prop(self, baton, *args, **kwargs):
+    def change_dir_prop(self, baton, name, value, pool=None):
         assert self.recent_baton[-1] == baton
         return svn.delta.editor_invoke_change_dir_prop(self.editor, baton, 
-                                                       *args, **kwargs)
+                                                       name, value, pool)
 
     @convert_svn_error
     def delete_entry(self, *args, **kwargs):
@@ -132,10 +177,10 @@ class Editor:
         return baton
 
     @convert_svn_error
-    def change_file_prop(self, baton, *args, **kwargs):
+    def change_file_prop(self, baton, name, value, pool=None):
         assert self.recent_baton[-1] == baton
-        svn.delta.editor_invoke_change_file_prop(self.editor, baton, *args, 
-                                                 **kwargs)
+        svn.delta.editor_invoke_change_file_prop(self.editor, baton, name, 
+                                                 value, pool)
 
     @convert_svn_error
     def close_file(self, baton, *args, **kwargs):
@@ -177,10 +222,7 @@ class SvnRaTransport(Transport):
         self._backing_url = _backing_url.rstrip("/")
         Transport.__init__(self, bzr_url)
 
-        self._client = svn.client.create_context(self.pool)
-        self._client.auth_baton = _create_auth_baton(self.pool)
-        self._client.config = svn_config
-
+        self._client = create_svn_client(self.pool)
         try:
             self.mutter('opening SVN RA connection to %r' % self._backing_url)
             self._ra = svn.client.open_ra_session(self._backing_url.encode('utf8'), 
@@ -188,19 +230,32 @@ class SvnRaTransport(Transport):
         except SubversionException, (_, num):
             if num in (svn.core.SVN_ERR_RA_SVN_REPOS_NOT_FOUND,):
                 raise NoSvnRepositoryPresent(url=url)
+            if num == svn.core.SVN_ERR_BAD_URL:
+                raise InvalidURL(url)
             raise
 
         from bzrlib.plugins.svn import lazy_check_versions
         lazy_check_versions()
 
+        self._busy = False
+
+    def _mark_busy(self):
+        assert not self._busy
+        self._busy = True
+
+    def _unmark_busy(self):
+        assert self._busy
+        self._busy = False
+
     def mutter(self, text):
         if 'transport' in debug.debug_flags:
             mutter(text)
 
     class Reporter:
-        def __init__(self, (reporter, report_baton)):
+        def __init__(self, transport, (reporter, report_baton)):
             self._reporter = reporter
             self._baton = report_baton
+            self._transport = transport
 
         @convert_svn_error
         def set_path(self, path, revnum, start_empty, lock_token, pool=None):
@@ -223,11 +278,13 @@ class SvnRaTransport(Transport):
         def finish_report(self, pool=None):
             svn.ra.reporter2_invoke_finish_report(self._reporter, 
                     self._baton, pool)
+            self._transport._unmark_busy()
 
         @convert_svn_error
         def abort_report(self, pool=None):
             svn.ra.reporter2_invoke_abort_report(self._reporter, 
                     self._baton, pool)
+            self._transport._unmark_busy()
 
     def has(self, relpath):
         """See Transport.has()."""
@@ -246,62 +303,122 @@ class SvnRaTransport(Transport):
         raise TransportNotPossible('stat not supported on Subversion')
 
     @convert_svn_error
+    @needs_busy
     def get_uuid(self):
         self.mutter('svn get-uuid')
         return svn.ra.get_uuid(self._ra)
 
-    @convert_svn_error
     def get_repos_root(self):
+        root = self.get_svn_repos_root()
+        if (self.base.startswith("svn+http:") or 
+            self.base.startswith("svn+https:")):
+            return "svn+%s" % root
+        return root
+
+    @convert_svn_error
+    @needs_busy
+    def get_svn_repos_root(self):
         if self._root is None:
             self.mutter("svn get-repos-root")
             self._root = svn.ra.get_repos_root(self._ra)
         return self._root
 
     @convert_svn_error
+    @needs_busy
     def get_latest_revnum(self):
         self.mutter("svn get-latest-revnum")
         return svn.ra.get_latest_revnum(self._ra)
 
-    @convert_svn_error
-    def do_switch(self, switch_rev, recurse, switch_url, *args, **kwargs):
-        assert self._backing_url == self.svn_url, "backing url invalid: %r != %r" % (self._backing_url, self.svn_url)
-        self.mutter('svn switch -r %d -> %r' % (switch_rev, switch_url))
-        return self.Reporter(svn.ra.do_switch(self._ra, switch_rev, "", recurse, switch_url, *args, **kwargs))
+    def _make_editor(self, editor, pool=None):
+        edit, edit_baton = svn.delta.make_editor(editor, pool)
+        self._edit = edit
+        self._edit_baton = edit_baton
+        return self._edit, self._edit_baton
 
     @convert_svn_error
-    def get_log(self, path, from_revnum, to_revnum, *args, **kwargs):
+    def do_switch(self, switch_rev, recurse, switch_url, editor, pool=None):
+        self._open_real_transport()
+        self.mutter('svn switch -r %d -> %r' % (switch_rev, switch_url))
+        self._mark_busy()
+        edit, edit_baton = self._make_editor(editor, pool)
+        return self.Reporter(self, svn.ra.do_switch(self._ra, switch_rev, "", 
+                             recurse, switch_url, edit, edit_baton, pool))
+
+    @convert_svn_error
+    @needs_busy
+    def get_log(self, path, from_revnum, to_revnum, limit, discover_changed_paths, 
+                strict_node_history, revprops, rcvr, pool=None):
         self.mutter('svn log %r:%r %r' % (from_revnum, to_revnum, path))
-        return svn.ra.get_log(self._ra, [self._request_path(path)], from_revnum, to_revnum, *args, **kwargs)
+        if hasattr(svn.ra, 'get_log2'):
+            return svn.ra.get_log2(self._ra, [self._request_path(path)], 
+                           from_revnum, to_revnum, limit, discover_changed_paths,
+                           strict_node_history, False, 
+                           revprops, rcvr, pool)
+
+        class LogEntry:
+            def __init__(self, changed_paths, rev, author, date, message):
+                self.changed_paths = changed_paths
+                self.revprops = {}
+                if svn.core.SVN_PROP_REVISION_AUTHOR in revprops:
+                    self.revprops[svn.core.SVN_PROP_REVISION_AUTHOR] = author
+                if svn.core.SVN_PROP_REVISION_LOG in revprops:
+                    self.revprops[svn.core.SVN_PROP_REVISION_LOG] = message
+                if svn.core.SVN_PROP_REVISION_DATE in revprops:
+                    self.revprops[svn.core.SVN_PROP_REVISION_DATE] = date
+                # FIXME: Check other revprops
+                # FIXME: Handle revprops is None
+                self.revision = rev
+                self.has_children = None
+
+        def rcvr_convert(orig_paths, rev, author, date, message, pool):
+            rcvr(LogEntry(orig_paths, rev, author, date, message), pool)
+
+        return svn.ra.get_log(self._ra, [self._request_path(path)], 
+                              from_revnum, to_revnum, limit, discover_changed_paths, 
+                              strict_node_history, rcvr_convert, pool)
+
+    def _open_real_transport(self):
+        if self._backing_url != self.svn_url:
+            self.reparent(self.base)
+        assert self._backing_url == self.svn_url
 
     def reparent_root(self):
         if self._is_http_transport():
-            self.svn_url = self.base = self.get_repos_root()
+            self.svn_url = self.get_svn_repos_root()
+            self.base = self.get_repos_root()
         else:
             self.reparent(self.get_repos_root())
 
     @convert_svn_error
+    def change_rev_prop(self, revnum, name, value, pool=None):
+        self.mutter('svn revprop -r%d --set %s=%s' % (revnum, name, value))
+        svn.ra.change_rev_prop(self._ra, revnum, name, value)
+
+    @convert_svn_error
+    @needs_busy
     def reparent(self, url):
         url = url.rstrip("/")
-        if url == self.svn_url:
-            return
         self.base = url
-        self.svn_url = url
-        self._backing_url = url
+        self.svn_url = bzr_to_svn_url(url)
+        if self.svn_url == self._backing_url:
+            return
         if hasattr(svn.ra, 'reparent'):
             self.mutter('svn reparent %r' % url)
-            svn.ra.reparent(self._ra, url, self.pool)
+            svn.ra.reparent(self._ra, self.svn_url, self.pool)
         else:
             self.mutter('svn reparent (reconnect) %r' % url)
             self._ra = svn.client.open_ra_session(self.svn_url.encode('utf8'), 
                     self._client, self.pool)
+        self._backing_url = self.svn_url
 
     @convert_svn_error
+    @needs_busy
     def get_dir(self, path, revnum, pool=None, kind=False):
         self.mutter("svn ls -r %d '%r'" % (revnum, path))
+        assert len(path) == 0 or path[0] != "/"
         path = self._request_path(path)
         # ra_dav backends fail with strange errors if the path starts with a 
         # slash while other backends don't.
-        assert len(path) == 0 or path[0] != "/"
         if hasattr(svn.ra, 'get_dir2'):
             fields = 0
             if kind:
@@ -311,12 +428,13 @@ class SvnRaTransport(Transport):
             return svn.ra.get_dir(self._ra, path, revnum)
 
     def _request_path(self, relpath):
-        if self._backing_url != self.svn_url:
-            relpath = urlutils.join(
-                    urlutils.relative_url(self._backing_url, self.svn_url),
-                    relpath)
-        relpath = relpath.rstrip("/")
-        return relpath
+        if self._backing_url == self.svn_url:
+            return relpath.strip("/")
+        newrelpath = urlutils.join(
+                urlutils.relative_url(self._backing_url+"/", self.svn_url+"/"),
+                relpath).strip("/")
+        self.mutter('request path %r -> %r' % (relpath, newrelpath))
+        return newrelpath
 
     @convert_svn_error
     def list_dir(self, relpath):
@@ -333,6 +451,7 @@ class SvnRaTransport(Transport):
         return dirents.keys()
 
     @convert_svn_error
+    @needs_busy
     def get_lock(self, path):
         return svn.ra.get_lock(self._ra, path)
 
@@ -344,14 +463,15 @@ class SvnRaTransport(Transport):
         def unlock(self):
             self.transport.unlock(self.locks)
 
-
     @convert_svn_error
+    @needs_busy
     def unlock(self, locks, break_lock=False):
         def lock_cb(baton, path, do_lock, lock, ra_err, pool):
             pass
         return svn.ra.unlock(self._ra, locks, break_lock, lock_cb)
 
     @convert_svn_error
+    @needs_busy
     def lock_write(self, path_revs, comment=None, steal_lock=False):
         return self.PhonyLock() # FIXME
         tokens = {}
@@ -361,13 +481,15 @@ class SvnRaTransport(Transport):
         return SvnLock(self, tokens)
 
     @convert_svn_error
+    @needs_busy
     def check_path(self, path, revnum, *args, **kwargs):
-        path = self._request_path(path)
         assert len(path) == 0 or path[0] != "/"
+        path = self._request_path(path)
         self.mutter("svn check_path -r%d %s" % (revnum, path))
         return svn.ra.check_path(self._ra, path.encode('utf-8'), revnum, *args, **kwargs)
 
     @convert_svn_error
+    @needs_busy
     def mkdir(self, relpath, mode=None):
         assert len(relpath) == 0 or relpath[0] != "/"
         path = urlutils.join(self.svn_url, relpath)
@@ -381,15 +503,51 @@ class SvnRaTransport(Transport):
             raise
 
     @convert_svn_error
-    def do_update(self, revnum, *args, **kwargs):
-        assert self._backing_url == self.svn_url, "backing url invalid: %r != %r" % (self._backing_url, self.svn_url)
-        self.mutter('svn update -r %r' % revnum)
-        return self.Reporter(svn.ra.do_update(self._ra, revnum, "", *args, **kwargs))
+    def replay(self, revision, low_water_mark, send_deltas, editor, pool=None):
+        self._open_real_transport()
+        self.mutter('svn replay -r%r:%r' % (low_water_mark, revision))
+        self._mark_busy()
+        edit, edit_baton = self._make_editor(editor, pool)
+        svn.ra.replay(self._ra, revision, low_water_mark, send_deltas,
+                      edit, edit_baton, pool)
 
     @convert_svn_error
-    def get_commit_editor(self, *args, **kwargs):
-        assert self._backing_url == self.svn_url, "backing url invalid: %r != %r" % (self._backing_url, self.svn_url)
-        return Editor(svn.ra.get_commit_editor(self._ra, *args, **kwargs))
+    def do_update(self, revnum, recurse, editor, pool=None):
+        self._open_real_transport()
+        self.mutter('svn update -r %r' % revnum)
+        self._mark_busy()
+        edit, edit_baton = self._make_editor(editor, pool)
+        return self.Reporter(self, svn.ra.do_update(self._ra, revnum, "", 
+                             recurse, edit, edit_baton, pool))
+
+    @convert_svn_error
+    def has_capability(self, cap):
+        return svn.ra.has_capability(self._ra, cap)
+
+    @convert_svn_error
+    def revprop_list(self, revnum, pool=None):
+        self.mutter('svn revprop-list -r %r' % revnum)
+        return svn.ra.rev_proplist(self._ra, revnum, pool)
+
+    @convert_svn_error
+    def get_commit_editor(self, revprops, done_cb, lock_token, keep_locks):
+        self._open_real_transport()
+        self._mark_busy()
+        try:
+            if hasattr(svn.ra, 'get_commit_editor3'):
+                editor = svn.ra.get_commit_editor3(self._ra, revprops, done_cb, 
+                                                  lock_token, keep_locks)
+            elif revprops.keys() != [svn.core.SVN_PROP_REVISION_LOG]:
+                raise NotImplementedError()
+            else:
+                editor = svn.ra.get_commit_editor2(self._ra, 
+                            revprops[svn.core.SVN_PROP_REVISION_LOG],
+                            done_cb, lock_token, keep_locks)
+
+            return Editor(self, editor)
+        except:
+            self._unmark_busy()
+            raise
 
     def listable(self):
         """See Transport.listable().
@@ -413,7 +571,8 @@ class SvnRaTransport(Transport):
 
     def clone_root(self):
         if self._is_http_transport():
-            return SvnRaTransport(self.get_repos_root(), self.base)
+            return SvnRaTransport(self.get_repos_root(), 
+                                  bzr_to_svn_url(self.base))
         return SvnRaTransport(self.get_repos_root())
 
     def clone(self, offset=None):
