@@ -20,9 +20,7 @@ from bzrlib.trace import info, mutter
 import bzrlib.ui as ui
 
 from errors import RebaseNotPresent
-from revids import (generate_svn_revision_id, parse_svn_revision_id, 
-                    MAPPING_VERSION,  unescape_svn_path)
-from scheme import BranchingScheme, guess_scheme_from_branch_path
+from mapping import parse_revision_id
 
 class UpgradeChangesContent(BzrError):
     """Inconsistency was found upgrading the mapping of a revision."""
@@ -32,40 +30,8 @@ class UpgradeChangesContent(BzrError):
         self.revid = revid
 
 
-def parse_legacy_revision_id(revid):
-    """Try to parse a legacy Subversion revision id.
-    
-    :param revid: Revision id to parse
-    :return: tuple with (uuid, branch_path, revision number, scheme, mapping version)
-    """
-    if revid.startswith("svn-v1:"):
-        revid = revid[len("svn-v1:"):]
-        at = revid.index("@")
-        fash = revid.rindex("-")
-        uuid = revid[at+1:fash]
-        branch_path = unescape_svn_path(revid[fash+1:])
-        revnum = int(revid[0:at])
-        assert revnum >= 0
-        return (uuid, branch_path, revnum, None, 1)
-    elif revid.startswith("svn-v2:"):
-        revid = revid[len("svn-v2:"):]
-        at = revid.index("@")
-        fash = revid.rindex("-")
-        uuid = revid[at+1:fash]
-        branch_path = unescape_svn_path(revid[fash+1:])
-        revnum = int(revid[0:at])
-        assert revnum >= 0
-        return (uuid, branch_path, revnum, None, 2)
-    elif revid.startswith("svn-v3-"):
-        (uuid, bp, rev, scheme) = parse_svn_revision_id(revid)
-        if scheme == "undefined":
-            scheme = None
-        return (uuid, bp, rev, scheme, 3)
 
-    raise InvalidRevisionId(revid, None)
-
-
-def create_upgraded_revid(revid):
+def create_upgraded_revid(revid, mapping_suffix, upgrade_suffix="-upgrade"):
     """Create a new revision id for an upgraded version of a revision.
     
     Prevents suffix to be appended needlessly.
@@ -73,18 +39,22 @@ def create_upgraded_revid(revid):
     :param revid: Original revision id.
     :return: New revision id
     """
-    suffix = "-svn%d-upgrade" % MAPPING_VERSION
-    if revid.endswith("-upgrade"):
-        return revid[0:revid.rfind("-svn")] + suffix
+    if revid.endswith(upgrade_suffix):
+        return revid[0:revid.rfind("-svn")] + mapping_suffix + upgrade_suffix
     else:
-        return revid + suffix
+        return revid + mapping_suffix + upgrade_suffix
 
 
 def upgrade_workingtree(wt, svn_repository, allow_changes=False, verbose=False):
-    upgrade_branch(wt.branch, svn_repository, allow_changes=allow_changes, verbose=verbose)
+    """Upgrade a working tree.
+
+    :param svn_repository: Subversion repository object
+    """
+    renames = upgrade_branch(wt.branch, svn_repository, allow_changes=allow_changes, verbose=verbose)
     last_revid = wt.branch.last_revision()
     wt.set_parent_trees([(last_revid, wt.branch.repository.revision_tree(last_revid))])
     # TODO: Should also adjust file ids in working tree if necessary
+    return renames
 
 
 def upgrade_branch(branch, svn_repository, allow_changes=False, verbose=False):
@@ -97,9 +67,10 @@ def upgrade_branch(branch, svn_repository, allow_changes=False, verbose=False):
     """
     revid = branch.last_revision()
     renames = upgrade_repository(branch.repository, svn_repository, 
-              revid, allow_changes=allow_changes, verbose=verbose)
+              revision_id=revid, allow_changes=allow_changes, verbose=verbose)
     if len(renames) > 0:
         branch.generate_revision_history(renames[revid])
+    return renames
 
 
 def check_revision_changed(oldrev, newrev):
@@ -114,7 +85,14 @@ def check_revision_changed(oldrev, newrev):
         raise UpgradeChangesContent(oldrev.revision_id)
 
 
-def generate_upgrade_map(revs):
+def generate_upgrade_map(new_mapping, revs):
+    """Generate an upgrade map for use by bzr-rebase.
+
+    :param new_mapping: BzrSvnMapping to upgrade revisions to.
+    :param revs: Iterator over revisions to upgrade.
+    :return: Map from old revids as keys, new revids as values stored in a 
+             dictionary.
+    """
     rename_map = {}
     pb = ui.ui_factory.nested_progress_bar()
     # Create a list of revisions that can be renamed during the upgade
@@ -122,39 +100,56 @@ def generate_upgrade_map(revs):
         for revid in revs:
             pb.update('gather revision information', revs.index(revid), len(revs))
             try:
-                (uuid, bp, rev, scheme, _) = parse_legacy_revision_id(revid)
+                (uuid, bp, rev, mapping) = parse_revision_id(revid)
             except InvalidRevisionId:
                 # Not a bzr-svn revision, nothing to do
                 continue
-            if scheme is None:
-                scheme = guess_scheme_from_branch_path(bp)
-            newrevid = generate_svn_revision_id(uuid, rev, bp, scheme)
+            newrevid = new_mapping.generate_revision_id(uuid, rev, bp)
+            if revid == newrevid:
+                continue
             rename_map[revid] = newrevid
     finally:
         pb.finished()
 
     return rename_map
 
+MIN_REBASE_VERSION = (0, 2)
 
-def create_upgrade_plan(repository, svn_repository, revision_id=None,
-                        allow_changes=False):
+def check_rebase_version():
+    """Check what version of bzr-rebase is installed.
+
+    Raises an exception when the version installed is older than 
+    MIN_REBASE_VERSION.
+
+    :raises RebaseNotPresent: Raised if bzr-rebase is not installed or too old.
+    """
+    try:
+        from bzrlib.plugins.rebase import version_info as rebase_version_info
+        if rebase_version_info[:2] < MIN_REBASE_VERSION:
+            raise RebaseNotPresent("Version %r present, at least %r required" 
+                                   % (rebase_version_info, MIN_REBASE_VERSION))
+    except ImportError, e:
+        raise RebaseNotPresent(e)
+
+
+def create_upgrade_plan(repository, svn_repository, new_mapping,
+                        revision_id=None, allow_changes=False):
     """Generate a rebase plan for upgrading revisions.
 
     :param repository: Repository to do upgrade in
     :param svn_repository: Subversion repository to fetch new revisions from.
+    :param new_mapping: New mapping to use.
     :param revision_id: Revision to upgrade (None for all revisions in 
         repository.)
     :param allow_changes: Whether an upgrade is allowed to change the contents
         of revisions.
     :return: Tuple with a rebase plan and map of renamed revisions.
     """
-    try:
-        from bzrlib.plugins.rebase.rebase import generate_transpose_plan
-    except ImportError, e:
-        raise RebaseNotPresent(e)
+    from bzrlib.plugins.rebase.rebase import generate_transpose_plan
+    check_rebase_version()
 
     graph = repository.get_revision_graph(revision_id)
-    upgrade_map = generate_upgrade_map(graph.keys())
+    upgrade_map = generate_upgrade_map(new_mapping, graph.keys())
    
     # Make sure all the required current version revisions are present
     for revid in upgrade_map.values():
@@ -168,8 +163,8 @@ def create_upgrade_plan(repository, svn_repository, revision_id=None,
             check_revision_changed(oldrev, newrev)
 
     plan = generate_transpose_plan(graph, upgrade_map, 
-                                   repository.revision_parents,
-                                   create_upgraded_revid)
+      repository.revision_parents,
+      lambda revid: create_upgraded_revid(revid, new_mapping.upgrade_suffix))
     def remove_parents((oldrevid, (newrevid, parents))):
         return (oldrevid, newrevid)
     upgrade_map.update(dict(map(remove_parents, plan.items())))
@@ -177,23 +172,25 @@ def create_upgrade_plan(repository, svn_repository, revision_id=None,
     return (plan, upgrade_map)
 
  
-def upgrade_repository(repository, svn_repository, revision_id=None, 
-                       allow_changes=False, verbose=False):
+def upgrade_repository(repository, svn_repository, new_mapping=None,
+                       revision_id=None, allow_changes=False, verbose=False):
     """Upgrade the revisions in repository until the specified stop revision.
 
     :param repository: Repository in which to upgrade.
     :param svn_repository: Repository to fetch new revisions from.
+    :param new_mapping: New mapping.
     :param revision_id: Revision id up until which to upgrade, or None for 
                         all revisions.
     :param allow_changes: Allow changes to mappings.
     :param verbose: Whether to print list of rewrites
     :return: Dictionary of mapped revisions
     """
-    try:
-        from bzrlib.plugins.rebase.rebase import (
-            replay_snapshot, rebase, rebase_todo)
-    except ImportError, e:
-        raise RebaseNotPresent(e)
+    check_rebase_version()
+    from bzrlib.plugins.rebase.rebase import (
+        replay_snapshot, rebase, rebase_todo)
+
+    if new_mapping is None:
+        new_mapping = svn_repository.get_mapping()
 
     # Find revisions that need to be upgraded, create
     # dictionary with revision ids in key, new parents in value
@@ -201,6 +198,7 @@ def upgrade_repository(repository, svn_repository, revision_id=None,
         repository.lock_write()
         svn_repository.lock_read()
         (plan, revid_renames) = create_upgrade_plan(repository, svn_repository, 
+                                                    new_mapping,
                                                     revision_id=revision_id,
                                                     allow_changes=allow_changes)
         if verbose:
@@ -208,12 +206,10 @@ def upgrade_repository(repository, svn_repository, revision_id=None,
                 info("%s -> %s" % (revid, plan[revid][0]))
         def fix_revid(revid):
             try:
-                (uuid, bp, rev, scheme, _) = parse_legacy_revision_id(revid)
+                (uuid, bp, rev, mapping) = parse_revision_id(revid)
             except InvalidRevisionId:
                 return revid
-            if scheme is None:
-                scheme = guess_scheme_from_branch_path(bp)
-            return generate_svn_revision_id(uuid, rev, bp, scheme)
+            return new_mapping.generate_revision_id(uuid, rev, bp)
         def replay(repository, oldrevid, newrevid, new_parents):
             return replay_snapshot(repository, oldrevid, newrevid, new_parents,
                                    revid_renames, fix_revid)
