@@ -20,11 +20,12 @@ from bzrlib import urlutils
 from bzrlib.branch import PullResult
 from bzrlib.bzrdir import BzrDirFormat, BzrDir
 from bzrlib.errors import (InvalidRevisionId, NotBranchError, NoSuchFile,
-                           NoRepositoryPresent, BzrError)
+                           NoRepositoryPresent, BzrError, UninitializableFormat,
+                           OutOfDateTree)
 from bzrlib.inventory import Inventory, InventoryFile, InventoryLink
 from bzrlib.lockable_files import TransportLock, LockableFiles
 from bzrlib.lockdir import LockDir
-from bzrlib.osutils import file_kind, fingerprint_file
+from bzrlib.osutils import file_kind, fingerprint_file, supports_executable
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 from bzrlib.tree import RevisionTree
@@ -32,16 +33,19 @@ from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import WorkingTree, WorkingTreeFormat
 
 from branch import SvnBranch
+from commit import _revision_id_to_svk_feature
 from convert import SvnConverter
 from errors import LocalCommitsUnsupported, NoSvnRepositoryPresent
-from repository import (SvnRepository, SVN_PROP_BZR_ANCESTRY,
-                        SVN_PROP_SVK_MERGE, SVN_PROP_BZR_FILEIDS, 
-                        SVN_PROP_BZR_REVISION_ID, SVN_PROP_BZR_REVISION_INFO,
-                        revision_id_to_svk_feature, generate_revision_metadata) 
-from revids import escape_svn_path
+from mapping import (SVN_PROP_BZR_ANCESTRY, SVN_PROP_BZR_FILEIDS, 
+                     SVN_PROP_BZR_REVISION_ID, SVN_PROP_BZR_REVISION_INFO,
+                     generate_revision_metadata)
+from remote import SvnRemoteAccess
+from repository import SvnRepository
+from svk import SVN_PROP_SVK_MERGE, parse_svk_features, serialize_svk_features
+from mapping import escape_svn_path
 from scheme import BranchingScheme
-from transport import (SvnRaTransport, svn_config, bzr_to_svn_url, 
-                       _create_auth_baton) 
+from transport import (SvnRaTransport, bzr_to_svn_url, create_svn_client,
+                       svn_config) 
 from tree import SvnBasisTree
 
 import os
@@ -70,11 +74,9 @@ class SvnWorkingTree(WorkingTree):
         self._branch = branch
         self.base_revnum = 0
         self.pool = Pool()
-        self.client_ctx = svn.client.create_context()
-        self.client_ctx.config = svn_config
+        self.client_ctx = create_svn_client(self.pool)
         self.client_ctx.log_msg_func2 = \
                 svn.client.svn_swig_py_get_commit_log_func
-        self.client_ctx.auth_baton = _create_auth_baton(self.pool)
 
         self._get_wc()
         status = svn.wc.revision_status(self.basedir, None, True, None, None)
@@ -99,7 +101,7 @@ class SvnWorkingTree(WorkingTree):
             os.makedirs(os.path.join(self.controldir, 'lock'))
         except OSError:
             pass
-        control_transport = bzrdir.transport.clone(os.path.join(
+        control_transport = bzrdir.transport.clone(urlutils.join(
                                                    svn.wc.get_adm_dir(), 'bzr'))
         self._control_files = LockableFiles(control_transport, 'lock', LockDir)
 
@@ -107,12 +109,12 @@ class SvnWorkingTree(WorkingTree):
         ignores = set([svn.wc.get_adm_dir()])
         ignores.update(svn.wc.get_default_ignores(svn_config))
 
-        def dir_add(wc, prefix):
+        def dir_add(wc, prefix, patprefix):
             ignorestr = svn.wc.prop_get(svn.core.SVN_PROP_IGNORE, 
                                         self.abspath(prefix).rstrip("/"), wc)
             if ignorestr is not None:
                 for pat in ignorestr.splitlines():
-                    ignores.add("./"+os.path.join(prefix, pat))
+                    ignores.add(urlutils.joinpath(patprefix, pat))
 
             entries = svn.wc.entries_read(wc, False)
             for entry in entries:
@@ -127,13 +129,13 @@ class SvnWorkingTree(WorkingTree):
                 subwc = svn.wc.adm_open3(wc, self.abspath(subprefix), False, 
                                          0, None)
                 try:
-                    dir_add(subwc, subprefix)
+                    dir_add(subwc, subprefix, urlutils.joinpath(patprefix, entry))
                 finally:
                     svn.wc.adm_close(subwc)
 
         wc = self._get_wc()
         try:
-            dir_add(wc, "")
+            dir_add(wc, "", ".")
         finally:
             svn.wc.adm_close(wc)
 
@@ -222,13 +224,13 @@ class SvnWorkingTree(WorkingTree):
         """Generate a bzr file id from a Subversion file name. 
         
         :param revnum: Revision number.
-        :param path: Absolute path.
+        :param path: Absolute path within the Subversion repository.
         :return: Tuple with file id and revision id.
         """
         assert isinstance(revnum, int) and revnum >= 0
-        assert isinstance(path, basestring)
+        assert isinstance(path, str)
 
-        (_, rp) = self.branch.scheme.unprefix(path)
+        rp = self.branch.unprefix(path)
         entry = self.base_tree.id_map[rp]
         assert entry[0] is not None
         assert isinstance(entry[0], str), "fileid %r for %r is not a string" % (entry[0], path)
@@ -239,6 +241,7 @@ class SvnWorkingTree(WorkingTree):
 
         def add_file_to_inv(relpath, id, revid, parent_id):
             """Add a file to the inventory."""
+            assert isinstance(relpath, unicode)
             if os.path.islink(self.abspath(relpath)):
                 file = InventoryLink(id, os.path.basename(relpath), parent_id)
                 file.revision = revid
@@ -304,6 +307,7 @@ class SvnWorkingTree(WorkingTree):
                 return ("NEW-" + escape_svn_path(entry.url[len(entry.repos):].strip("/")), None)
 
         def add_dir_to_inv(relpath, wc, parent_id):
+            assert isinstance(relpath, unicode)
             entries = svn.wc.entries_read(wc, False)
             entry = entries[""]
             assert parent_id is None or isinstance(parent_id, str), \
@@ -316,7 +320,7 @@ class SvnWorkingTree(WorkingTree):
             assert isinstance(id, str), "%r is not a string" % id
 
             # First handle directory itself
-            inv.add_path(relpath, 'directory', id, parent_id).revision = revid
+            inv.add_path(relpath.decode("utf-8"), 'directory', id, parent_id).revision = revid
             if relpath == "":
                 inv.revision_id = revid
 
@@ -324,7 +328,7 @@ class SvnWorkingTree(WorkingTree):
                 if name == "":
                     continue
 
-                subrelpath = os.path.join(relpath, name)
+                subrelpath = os.path.join(relpath, name.decode("utf-8"))
 
                 entry = entries[name]
                 assert entry
@@ -345,7 +349,7 @@ class SvnWorkingTree(WorkingTree):
 
         rootwc = self._get_wc() 
         try:
-            add_dir_to_inv("", rootwc, None)
+            add_dir_to_inv(u"", rootwc, None)
         finally:
             svn.wc.adm_close(rootwc)
 
@@ -405,7 +409,9 @@ class SvnWorkingTree(WorkingTree):
     def commit(self, message=None, message_callback=None, revprops=None, 
                timestamp=None, timezone=None, committer=None, rev_id=None, 
                allow_pointless=True, strict=False, verbose=False, local=False, 
-               reporter=None, config=None, specific_files=None):
+               reporter=None, config=None, specific_files=None, author=None):
+        if author is not None:
+            revprops['author'] = author
         # FIXME: Use allow_pointless
         # FIXME: Use verbose
         # FIXME: Use reporter
@@ -435,8 +441,8 @@ class SvnWorkingTree(WorkingTree):
             extra = ""
         wc = self._get_wc(write_lock=True)
         try:
-            svn.wc.prop_set(SVN_PROP_BZR_REVISION_ID+str(self.branch.scheme), 
-                             self._get_bzr_revids() + extra,
+            svn.wc.prop_set(SVN_PROP_BZR_REVISION_ID+str(self.branch.mapping.scheme), 
+                             self._get_bzr_revids(self._get_base_branch_props()) + extra,
                              self.basedir, wc)
             svn.wc.prop_set(SVN_PROP_BZR_REVISION_INFO, 
                              generate_revision_metadata(timestamp, 
@@ -448,20 +454,23 @@ class SvnWorkingTree(WorkingTree):
             svn.wc.adm_close(wc)
 
         try:
-            commit_info = svn.client.commit3(specific_files, True, False, 
-                                         self.client_ctx)
+            try:
+                commit_info = svn.client.commit3(specific_files, True, False, 
+                                                 self.client_ctx)
+            except SubversionException, (_, num):
+                if num == svn.core.SVN_ERR_FS_TXN_OUT_OF_DATE:
+                    raise OutOfDateTree(self)
+                raise
         except:
             # Reset properties so the next subversion commit won't 
             # accidently set these properties.
             wc = self._get_wc(write_lock=True)
-            svn.wc.prop_set(SVN_PROP_BZR_REVISION_ID+str(self.branch.scheme), 
-                             self._get_bzr_revids(), self.basedir, wc)
+            base_branch_props = self._get_base_branch_props()
+            svn.wc.prop_set(SVN_PROP_BZR_REVISION_ID+str(self.branch.mapping.scheme), 
+                             self._get_bzr_revids(base_branch_props), self.basedir, wc)
             svn.wc.prop_set(SVN_PROP_BZR_REVISION_INFO, 
-                self.branch.repository.branchprop_list.get_property(
-                self.branch.get_branch_path(self.base_revnum), 
-                self.base_revnum, 
-                SVN_PROP_BZR_REVISION_INFO, ""), 
-                self.basedir, wc)
+                              base_branch_props.get(SVN_PROP_BZR_REVISION_INFO, ""),
+                              self.basedir, wc)
             svn.wc.adm_close(wc)
             raise
 
@@ -549,7 +558,7 @@ class SvnWorkingTree(WorkingTree):
         return self.base_tree
 
     def pull(self, source, overwrite=False, stop_revision=None, 
-             delta_reporter=None):
+             delta_reporter=None, possible_transports=None):
         # FIXME: Use delta_reporter
         # FIXME: Use overwrite
         result = PullResult()
@@ -591,31 +600,26 @@ class SvnWorkingTree(WorkingTree):
         if wc is None:
             svn.wc.adm_close(subwc)
 
+    def _get_base_branch_props(self):
+        return self.branch.repository.branchprop_list.get_properties(
+                self.branch.get_branch_path(self.base_revnum), self.base_revnum)
+
     def _get_new_file_ids(self, wc):
-        committed = self.branch.repository.branchprop_list.get_property(
-                self.branch.get_branch_path(self.base_revnum), self.base_revnum, 
-                SVN_PROP_BZR_FILEIDS, "")
+        committed = self._get_base_branch_props().get(SVN_PROP_BZR_FILEIDS, "")
         existing = svn.wc.prop_get(SVN_PROP_BZR_FILEIDS, self.basedir, wc)
         if existing is None or committed == existing:
             return {}
         return dict(map(lambda x: str(x).split("\t"), 
             existing.splitlines()))
 
-    def _get_bzr_revids(self):
-        return self.branch.repository.branchprop_list.get_property(
-                self.branch.get_branch_path(self.base_revnum), 
-                self.base_revnum, 
-                SVN_PROP_BZR_REVISION_ID+str(self.branch.scheme), "")
+    def _get_bzr_revids(self, base_branch_props):
+        return base_branch_props.get(SVN_PROP_BZR_REVISION_ID+str(self.branch.mapping.scheme), "")
 
-    def _get_bzr_merges(self):
-        return self.branch.repository.branchprop_list.get_property(
-                self.branch.get_branch_path(self.base_revnum), 
-                self.base_revnum, SVN_PROP_BZR_ANCESTRY+str(self.branch.scheme), "")
+    def _get_bzr_merges(self, base_branch_props):
+        return base_branch_props.get(SVN_PROP_BZR_ANCESTRY+str(self.branch.mapping.scheme), "")
 
-    def _get_svk_merges(self):
-        return self.branch.repository.branchprop_list.get_property(
-                self.branch.get_branch_path(self.base_revnum), 
-                self.base_revnum, SVN_PROP_SVK_MERGE, "")
+    def _get_svk_merges(self, base_branch_props):
+        return base_branch_props.get(SVN_PROP_SVK_MERGE, "")
 
     def set_pending_merges(self, merges):
         """See MutableTree.set_pending_merges()."""
@@ -627,20 +631,21 @@ class SvnWorkingTree(WorkingTree):
             else:
                 bzr_merge = ""
 
-            svn.wc.prop_set(SVN_PROP_BZR_ANCESTRY+str(self.branch.scheme), 
-                                 self._get_bzr_merges() + bzr_merge, 
+            svn.wc.prop_set(SVN_PROP_BZR_ANCESTRY+str(self.branch.mapping.scheme), 
+                                 self._get_bzr_merges(self._get_base_branch_props()) + bzr_merge, 
                                  self.basedir, wc)
+            
+            svk_merges = parse_svk_features(self._get_svk_merges(self._get_base_branch_props()))
 
             # Set svk:merge
-            svk_merge = ""
             for merge in merges:
                 try:
-                    svk_merge += revision_id_to_svk_feature(merge) + "\n"
+                    svk_merges.add(_revision_id_to_svk_feature(merge))
                 except InvalidRevisionId:
                     pass
 
             svn.wc.prop_set2(SVN_PROP_SVK_MERGE, 
-                             self._get_svk_merges() + svk_merge, self.basedir, 
+                             serialize_svk_features(svk_merges), self.basedir, 
                              wc, False)
         finally:
             svn.wc.adm_close(wc)
@@ -651,11 +656,11 @@ class SvnWorkingTree(WorkingTree):
         self.set_pending_merges(merges)
 
     def pending_merges(self):
-        merged = self._get_bzr_merges().splitlines()
+        merged = self._get_bzr_merges(self._get_base_branch_props()).splitlines()
         wc = self._get_wc()
         try:
             merged_data = svn.wc.prop_get(
-                SVN_PROP_BZR_ANCESTRY+str(self.branch.scheme), self.basedir, wc)
+                SVN_PROP_BZR_ANCESTRY+str(self.branch.mapping.scheme), self.basedir, wc)
             if merged_data is None:
                 set_merged = []
             else:
@@ -675,12 +680,22 @@ class SvnWorkingTree(WorkingTree):
         pass
 
     def unlock(self):
+        # non-implementation specific cleanup
+        self._cleanup()
+
         # reverse order of locking.
         try:
             return self._control_files.unlock()
         finally:
             self.branch.unlock()
 
+    if not supports_executable():
+        def is_executable(self, file_id, path=None):
+            inv = self.basis_tree()._inventory
+            if file_id in inv:
+                return inv[file_id].executable
+            # Default to not executable
+            return False
 
 
 class SvnWorkingTreeFormat(WorkingTreeFormat):
@@ -718,10 +733,9 @@ class SvnCheckout(BzrDir):
             svn.wc.adm_close(wc)
 
         remote_transport = SvnRaTransport(svn_url)
-        self.svn_root_transport = SvnRaTransport(remote_transport.get_repos_root())
+        self.remote_bzrdir = SvnRemoteAccess(remote_transport)
+        self.svn_root_transport = remote_transport.clone_root()
         self.root_transport = self.transport = transport
-
-        self.branch_path = svn_url[len(bzr_to_svn_url(self.svn_root_transport.base)):]
         
     def clone(self, path, revision_id=None, force_new_repo=False):
         raise NotImplementedError(self.clone)
@@ -730,24 +744,34 @@ class SvnCheckout(BzrDir):
         return SvnWorkingTree(self, self.local_path, self.open_branch())
 
     def sprout(self, url, revision_id=None, force_new_repo=False, 
-               recurse='down', possible_transports=None):
+               recurse='down', possible_transports=None, accelerator_tree=None,
+               hardlink=False):
         # FIXME: honor force_new_repo
         # FIXME: Use recurse
         result = get_rich_root_format().initialize(url)
-        repo = self.find_repository()
+        repo = self._find_repository()
         repo.clone(result, revision_id)
         branch = self.open_branch()
         branch.sprout(result, revision_id)
-        result.create_workingtree()
+        result.create_workingtree(hardlink=hardlink)
         return result
 
     def open_repository(self):
         raise NoRepositoryPresent(self)
 
     def find_repository(self):
-        return SvnRepository(self, self.svn_root_transport, self.branch_path)
+        raise NoRepositoryPresent(self)
 
-    def create_workingtree(self, revision_id=None):
+    def _find_repository(self):
+        return SvnRepository(self, self.svn_root_transport, 
+                             self.remote_bzrdir.branch_path)
+
+    def needs_format_conversion(self, format=None):
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        return not isinstance(self._format, format.__class__)
+
+    def create_workingtree(self, revision_id=None, hardlink=None):
         """See BzrDir.create_workingtree().
 
         Not implemented for Subversion because having a .svn directory
@@ -761,56 +785,19 @@ class SvnCheckout(BzrDir):
 
     def open_branch(self, unsupported=True):
         """See BzrDir.open_branch()."""
-        repos = self.find_repository()
+        repos = self._find_repository()
 
         try:
-            branch = SvnBranch(self.root_transport.base, repos, 
-                               self.branch_path)
+            branch = SvnBranch(self.svn_root_transport.base, repos, 
+                               self.remote_bzrdir.branch_path)
         except SubversionException, (_, num):
             if num == svn.core.SVN_ERR_WC_NOT_DIRECTORY:
                 raise NotBranchError(path=self.base)
             raise
+
+        branch.bzrdir = self.remote_bzrdir
  
-        branch.bzrdir = self
         return branch
 
 
-class SvnWorkingTreeDirFormat(BzrDirFormat):
-    """Working Tree implementation that uses Subversion working copies."""
-    _lock_class = TransportLock
 
-    @classmethod
-    def probe_transport(klass, transport):
-        format = klass()
-
-        if isinstance(transport, LocalTransport) and \
-            transport.has(svn.wc.get_adm_dir()):
-            return format
-
-        raise NotBranchError(path=transport.base)
-
-    def _open(self, transport):
-        subr_version = svn.core.svn_subr_version()
-        if subr_version.major == 1 and subr_version.minor < 4:
-            raise NoCheckoutSupport()
-        try:
-            return SvnCheckout(transport, self)
-        except SubversionException, (_, num):
-            if num in (svn.core.SVN_ERR_RA_LOCAL_REPOS_OPEN_FAILED,):
-                raise NoSvnRepositoryPresent(transport.base)
-            raise
-
-    def get_format_string(self):
-        return 'Subversion Local Checkout'
-
-    def get_format_description(self):
-        return 'Subversion Local Checkout'
-
-    def initialize_on_transport(self, transport):
-        raise NotImplementedError(self.initialize_on_transport)
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        if format is None:
-            format = get_rich_root_format()
-        return SvnConverter(format)
